@@ -24,6 +24,11 @@ else
 	DEBUG=false
 fi
 
+[ -n "${GREP}" ] || { GREP="`command -v grep`" && [ x"${GREP}" != x ] || { echo "$0: FAILED to locate GREP tool" >&2 ; exit 1 ; } ; export GREP ; }
+[ -n "${EGREP}" ] || { if ( [ x"`echo a | $GREP -E '(a|b)'`" = xa ] ) 2>/dev/null ; then EGREP="$GREP -E" ; else EGREP="`command -v egrep`" ; fi && [ x"${EGREP}" != x ] || { echo "$0: FAILED to locate EGREP tool" >&2 ; exit 1 ; } ; export EGREP ; }
+
+NUT_VERSION_QUERY=UPDATE_FILE "`dirname $0`"/tools/gitlog2version.sh
+
 if [ -n "${PYTHON-}" ] ; then
 	# May be a name/path of binary, or one with args - check both
 	(command -v "$PYTHON") \
@@ -106,8 +111,8 @@ if [ ! -f scripts/udev/nut-usbups.rules.in -o \
      ! -f scripts/devd/nut-usb.conf.in -o \
      ! -f scripts/devd/nut-usb.quirks -o \
      ! -f tools/nut-scanner/nutscan-usb.h ] \
-|| [ -n "`find drivers -newer scripts/hotplug/libhid.usermap | grep -E '(-hid|nutdrv_qx|usb.*)\.c'`" ] \
-|| [ -n "`find drivers -not -newer tools/nut-usbinfo.pl | grep -E '(-hid|nutdrv_qx|usb.*)\.c'`" ] \
+|| [ -n "`find drivers -newer scripts/hotplug/libhid.usermap | ${EGREP} '(-hid|nutdrv_qx|usb.*)\.c'`" ] \
+|| [ -n "`find drivers \! -newer tools/nut-usbinfo.pl | ${EGREP} '(-hid|nutdrv_qx|usb.*)\.c'`" ] \
 ; then
 	if perl -e 1; then
 		VERBOSE_FLAG_PERL=""
@@ -165,6 +170,18 @@ fi >&2
 [ -f NEWS ] || { echo "Please see NEWS.adoc for actual contents" > NEWS; }
 [ -f README ] || { echo "Please see README.adoc for actual contents" > README; }
 
+# Try to serve a fresh one at least when we remake from scratch like this
+# Note to not do it forcefully during `configure` or rebuild
+rm -f include/nut_version.h || true
+
+echo "----------------------------------------------------------------------"
+echo "Please note that on some systems the routine below can complain that "
+echo "  > configure.ac: warning: AC_INIT: not a literal: m4_esyscmd_s(...)"
+echo "but still does the right thing about PACKAGE_VERSION and PACKAGE_URL settings."
+echo "Check if your distro provides an 'autoconf-archive' package and if it helps."
+echo "Please post an issue in NUT bug tracker with platform details if it does not."
+echo "----------------------------------------------------------------------"
+
 echo "Calling autoreconf..."
 AUTOTOOL_RES=0
 if $DEBUG ; then
@@ -177,7 +194,8 @@ else
 fi
 
 [ "$AUTOTOOL_RES" = 0 ] && [ -s configure ] && [ -x configure ] \
-|| { cat << EOF
+|| { ( cat << EOF
+----------------------------------------------------------------------
 FAILED: did not generate an executable configure script!
 
 # Note: on some systems "autoreconf", "automake" et al are dispatcher
@@ -189,9 +207,11 @@ FAILED: did not generate an executable configure script!
 # check the configure.ac line it refers to and un-comment (or comment away)
 # the third argument for AM_SILENT_RULES check, or comment away the whole
 # "ifdef" block if your autotools still would not grok it.
+----------------------------------------------------------------------
 EOF
+	) >&2
 	exit 1
-} >&2
+}
 
 # Some autoconf versions may leave "/bin/sh" regardless of CONFIG_SHELL
 # which originally was made for "recheck" operations
@@ -217,15 +237,21 @@ else
 	CONFIG_SHELL="`head -1 configure | sed 's,^#!,,'`"
 fi
 
-# NOTE: Unquoted, may be multi-token
+echo "autogen.sh: testing generated script syntax with $CONFIG_SHELL" >&2
+# NOTE: Unquoted CONFIG_SHELL, may be multi-token
 $CONFIG_SHELL -n configure 2>/dev/null >/dev/null \
-|| { echo "FAILED: configure script did not pass shell interpreter syntax checks with $CONFIG_SHELL" >&2
+|| {
+	echo "----------------------------------------------------------------------" >&2
+	echo "FAILED: configure script did not pass shell interpreter syntax checks with $CONFIG_SHELL" >&2
 	echo "NOTE: If you are using an older OS release, try executing the script with" >&2
 	echo "a more functional shell implementation (dtksh, bash, dash...)" >&2
 	echo "You can re-run this script with a CONFIG_SHELL in environment" >&2
+	echo "----------------------------------------------------------------------" >&2
 	exit 1
 }
 
-echo "The generated configure script passed shell interpreter syntax checks"
+echo "----------------------------------------------------------------------"
+echo "SUCCESS: The generated configure script passed shell interpreter syntax checks"
 echo "Please proceed by running './configure --with-many-desired-options'"
 echo "For details check './configure --help' or docs/configure.txt in NUT sources"
+echo "----------------------------------------------------------------------"
