@@ -108,7 +108,7 @@
 #include "serial.h"
 
 #define DRIVER_NAME	"Bicker serial protocol"
-#define DRIVER_VERSION	"0.03"
+#define DRIVER_VERSION	"0.05"
 
 #define BICKER_SOH	0x01
 #define BICKER_EOT	0x04
@@ -646,7 +646,7 @@ static ssize_t bicker_delayed_shutdown(uint8_t seconds)
 
 	ret = bicker_receive_known(0x03, 0x32, &response, 1);
 	if (ret >= 0) {
-		upslogx(LOG_INFO, "Shutting down in %d seconds: response = 0x%02X",
+		upslogx(LOG_INSTCMD_POWERSTATE, "Shutting down in %d seconds: response = 0x%02X",
 			seconds, (unsigned)response);
 	}
 
@@ -671,13 +671,17 @@ static ssize_t bicker_shutdown(void)
 
 static int bicker_instcmd(const char *cmdname, const char *extra)
 {
+	/* May be used in logging below, but not as a command argument */
 	NUT_UNUSED_VARIABLE(extra);
+	upsdebug_INSTCMD_STARTING(cmdname, extra);
 
 	if (!strcasecmp(cmdname, "shutdown.return")) {
-		bicker_shutdown();
+		upslog_INSTCMD_POWERSTATE_CHANGE(cmdname, extra);
+		if (bicker_shutdown() >= 0)
+			return STAT_INSTCMD_HANDLED;
 	}
 
-	upslogx(LOG_NOTICE, "instcmd: unknown command [%s]", cmdname);
+	upslog_INSTCMD_UNKNOWN(cmdname, extra);
 	return STAT_INSTCMD_UNKNOWN;
 }
 
@@ -687,6 +691,8 @@ static int bicker_setvar(const char *varname, const char *val)
 	unsigned i;
 	BickerParameter parameter;
 
+	upsdebug_SET_STARTING(varname, val);
+
 	/* This should not be needed because when `bicker_write()` is
 	 * successful the `parameter` struct is populated but gcc seems
 	 * not to be smart enough to realize that and errors out with
@@ -718,7 +724,7 @@ static int bicker_setvar(const char *varname, const char *val)
 		}
 	}
 
-	upslogx(LOG_NOTICE, "setvar: unknown variable [%s]", varname);
+	upslog_SET_UNKNOWN(varname, val);
 	return STAT_SET_UNKNOWN;
 }
 
@@ -882,6 +888,11 @@ void upsdrv_help(void)
 {
 }
 
+/* optionally tweak prognames[] entries */
+void upsdrv_tweak_prognames(void)
+{
+}
+
 void upsdrv_makevartable(void)
 {
 }
