@@ -31,7 +31,7 @@
 #include "blazer.h"
 
 #define DRIVER_NAME	"Megatec/Q1 protocol serial driver"
-#define DRIVER_VERSION	"1.63"
+#define DRIVER_VERSION	"1.65"
 
 /* driver description structure */
 upsdrv_info_t upsdrv_info = {
@@ -109,6 +109,12 @@ void upsdrv_help(void)
 }
 
 
+/* optionally tweak prognames[] entries */
+void upsdrv_tweak_prognames(void)
+{
+}
+
+
 void upsdrv_makevartable(void)
 {
 	addvar(VAR_VALUE, "cablepower", "Set cable power for serial interface");
