@@ -27,7 +27,7 @@
 #include "attribute.h"
 
 #define DRIVER_NAME	"Liebert MultiLink UPS driver"
-#define DRIVER_VERSION	"1.04"
+#define DRIVER_VERSION	"1.07"
 
 /* driver description structure */
 upsdrv_info_t upsdrv_info = {
@@ -43,12 +43,16 @@ upsdrv_info_t upsdrv_info = {
 
 void upsdrv_shutdown(void)
 {
+	/* Only implement "shutdown.default"; do not invoke
+	 * general handling of other `sdcommands` here */
+
 	/* XXX: replace with a proper shutdown function (raise DTR) */
 
 	/* worse yet: stock cables don't support shutdown at all */
 
 	upslogx(LOG_ERR, "shutdown not supported");
-	set_exit_flag(-1);
+	if (handling_upsdrv_shutdown > 0)
+		set_exit_flag(EF_EXIT_FAILURE);
 }
 
 void upsdrv_initinfo(void)
@@ -107,7 +111,7 @@ void upsdrv_updateinfo(void)
 	else
 		ob_ctr++;
 
-	upsdebugx(2, "OB: state %d last %d now %d ctr %d",
+	upsdebugx(2, "OB: state %u last %u now %u ctr %u",
 		ob_state, ob_last, ob, ob_ctr);
 
 	if (ob_ctr >= DEBOUNCE) {
@@ -133,7 +137,7 @@ void upsdrv_updateinfo(void)
 	else
 		lb_ctr++;
 
-	upsdebugx(2, "LB: state %d last %d now %d ctr %d",
+	upsdebugx(2, "LB: state %u last %u now %u ctr %u",
 		lb_state, lb_last, lb, lb_ctr);
 
 	if (lb_ctr >= DEBOUNCE) {
@@ -175,6 +179,11 @@ void upsdrv_help(void)
 {
 }
 
+/* optionally tweak prognames[] entries */
+void upsdrv_tweak_prognames(void)
+{
+}
+
 void upsdrv_initups(void)
 {
 	upsfd = ser_open(device_path);
