@@ -28,7 +28,7 @@
 
 /* driver version */
 #define DRIVER_NAME	"'ATCL FOR UPS' USB driver"
-#define DRIVER_VERSION	"1.19"
+#define DRIVER_VERSION	"1.22"
 
 /* driver description structure */
 upsdrv_info_t upsdrv_info = {
@@ -183,6 +183,10 @@ static void usb_comm_fail(const char *fmt, ...)
 #ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_FORMAT_SECURITY
 #pragma GCC diagnostic ignored "-Wformat-security"
 #endif
+	/* Note: Not converting to hardened NUT methods with dynamic
+	 * format string checking, this one is used locally with
+	 * fixed strings (and args) */
+	/* FIXME: Actually, only fixed strings, no formatting here. */
 	ret = vsnprintf(why, sizeof(why), fmt, ap);
 #ifdef HAVE_PRAGMAS_FOR_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
 #pragma GCC diagnostic pop
@@ -555,11 +559,14 @@ void upsdrv_initups(void)
 			"Unable to find ATCL FOR UPS\n\n"
 
 			"Things to try:\n"
-			" - Connect UPS device to USB bus\n"
+			" - Connect UPS device to USB bus.\n"
 			" - Run this driver as another user (upsdrvctl -u or 'user=...' in ups.conf).\n"
-			"   See upsdrvctl(8) and ups.conf(5).\n\n"
+			"   See upsdrvctl(%s) and ups.conf(%s).\n"
+			" - Check with nutdrv_qx(%s) or blazer_usb(%s) driver instead.\n"
 
-			"Fatal error: unusable configuration");
+			"\nFatal error: unusable configuration",
+			MAN_SECTION_CMD_SYS, MAN_SECTION_CFG,
+			MAN_SECTION_CMD_SYS, MAN_SECTION_CMD_SYS);
 	}
 
 }
@@ -657,7 +664,9 @@ void upsdrv_updateinfo(void)
 static
 int instcmd(const char *cmdname, const char *extra)
 {
+	/* May be used in logging below, but not as a command argument */
 	NUT_UNUSED_VARIABLE(extra);
+	upsdebug_INSTCMD_STARTING(cmdname, extra);
 
 	/* FIXME: Which one is this - "load.off",
 	 * "shutdown.stayoff" or "shutdown.return"? */
@@ -677,6 +686,7 @@ int instcmd(const char *cmdname, const char *extra)
 			"%s: attempting to call usb_interrupt_write(01 00 00 00 00 00 00 00)",
 			__func__);
 
+		upslog_INSTCMD_POWERSTATE_CHANGE(cmdname, extra);
 		ret = usb_interrupt_write(udev,
 			SHUTDOWN_ENDPOINT, (usb_ctrl_charbuf)shutdown_packet,
 			SHUTDOWN_PACKETSIZE, ATCL_USB_TIMEOUT);
@@ -707,7 +717,7 @@ int instcmd(const char *cmdname, const char *extra)
 		return STAT_INSTCMD_HANDLED;
 	}
 
-	upslogx(LOG_NOTICE, "instcmd: unknown command [%s] [%s]", cmdname, extra);
+	upslog_INSTCMD_UNKNOWN(cmdname, extra);
 	return STAT_INSTCMD_UNKNOWN;
 }
 
@@ -728,6 +738,11 @@ void upsdrv_help(void)
 {
 }
 
+/* optionally tweak prognames[] entries */
+void upsdrv_tweak_prognames(void)
+{
+}
+
 void upsdrv_makevartable(void)
 {
 	/* NOTE: This driver uses a very custom device matching method,
