@@ -28,6 +28,9 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  *
  * History:
+ * - 2 August 2024.  Bill Elliot
+ * Just cleaned up formatting per current NUT standards.
+ *
  * - 7 February 2012.  Bill Elliot
  * Enhancing the driver for additional capabilities and later units.
  *
@@ -48,7 +51,7 @@ int setcmd(const char* varname, const char* setvalue);
 int instcmd(const char *cmdname, const char *extra);
 
 #define DRIVER_NAME	"Oneac EG/ON/OZ/OB UPS driver"
-#define DRIVER_VERSION	"0.84"
+#define DRIVER_VERSION	"0.86"
 
 /* driver description structure */
 upsdrv_info_t upsdrv_info = {
@@ -68,7 +71,7 @@ upsdrv_info_t upsdrv_info = {
 static char UpsFamily [3];
 
 /****************************************************************
- * Below are functions used only in this oneac driver			*
+ * Below are functions used only in this oneac driver           *
  ***************************************************************/
 
 /* Since an installed network card may delay responses from the UPS
@@ -83,15 +86,14 @@ static ssize_t OneacGetResponse (char* chBuff, const size_t BuffSize, int Expect
 
 	do
 	{
-		return_val = ser_get_line(upsfd,
-			chBuff, BuffSize, ENDCHAR, IGNCHARS, SECS, USEC);
+		return_val = ser_get_line(upsfd, chBuff, BuffSize, ENDCHAR, IGNCHARS, SECS, USEC);
 
 		if (return_val == ExpectedCount)
+		{
 			break;
+		}
 
-		upsdebugx (3,
-			"!OneacGetResponse retry (%" PRIiSIZE ", %d)...",
-			return_val, Retries);
+		upsdebugx (3, "!OneacGetResponse retry (%" PRIiSIZE ", %d)...", return_val, Retries);
 
 	} while (--Retries > 0);
 
@@ -105,7 +107,9 @@ static ssize_t OneacGetResponse (char* chBuff, const size_t BuffSize, int Expect
 	else
 	{
 		if (Retries < 10)
+		{
 			upsdebugx (2,"OneacGetResponse recovered (%d)...", Retries);
+		}
 
 		return_val = 0;					/* Good comms */
 	}
@@ -118,8 +122,11 @@ static void do_battery_test(void)
 	char buffer[32];
 
 	if (getval("testtime") == NULL)
+	{
 		snprintf(buffer, 3, "%s", DEFAULT_BAT_TEST_TIME);
-	else {
+	}
+	else
+	{
 		snprintf(buffer, 3, "%s", getval("testtime"));
 
 	/*the UPS wants this value to always be two characters long*/
@@ -130,7 +137,7 @@ static void do_battery_test(void)
 			buffer[0] = '0';
 		}
 	}
-	ser_send(upsfd,"%s%s%s",BAT_TEST_PREFIX,buffer,COMMAND_END);
+	ser_send(upsfd, "%s%s%s", BAT_TEST_PREFIX, buffer, COMMAND_END);
 }
 
 static int SetOutputAllow(const char* lowval, const char* highval)
@@ -150,12 +157,10 @@ static int SetOutputAllow(const char* lowval, const char* highval)
 		buffer[0] = '0';
 	}
 
-	upsdebugx (2,"SetOutputAllow sending %s%.3s,%.3s...",
-											SETX_OUT_ALLOW, buffer, highval);
+	upsdebugx (2,"SetOutputAllow sending %s%.3s,%.3s...", SETX_OUT_ALLOW, buffer, highval);
 
-	ser_send(upsfd,"%s%.3s,%.3s%s", SETX_OUT_ALLOW, buffer, highval,
-																COMMAND_END);
-	ser_get_line(upsfd,buffer,sizeof(buffer), ENDCHAR, IGNCHARS,SECS,USEC);
+	ser_send(upsfd, "%s%.3s,%.3s%s", SETX_OUT_ALLOW, buffer, highval, COMMAND_END);
+	ser_get_line(upsfd,buffer, sizeof(buffer), ENDCHAR, IGNCHARS, SECS,USEC);
 
 	if(buffer[0] == DONT_UNDERSTAND)
 	{
@@ -167,8 +172,7 @@ static int SetOutputAllow(const char* lowval, const char* highval)
 	return 0;						/* Valid command */
 }
 
-static void EliminateLeadingZeroes (const char* buff1, int StringSize, char* buff2,
-															const size_t buff2size)
+static void EliminateLeadingZeroes (const char* buff1, int StringSize, char* buff2, const size_t buff2size)
 {
 	int i = 0;
 	int j = 0;
@@ -189,7 +193,7 @@ static void EliminateLeadingZeroes (const char* buff1, int StringSize, char* buf
 
 
 /****************************************************************
- * Below are the commands that are called by main				*
+ * Below are the commands that are called by main               *
  ***************************************************************/
 
 void upsdrv_initups(void)
@@ -216,22 +220,20 @@ void upsdrv_initinfo(void)
 	 */
 
 	ser_flush_in(upsfd,"",0);
-	ser_send(upsfd,"%c%s",GET_FAMILY,COMMAND_END);
+	ser_send(upsfd,"%c%s", GET_FAMILY,COMMAND_END);
 
 	if(OneacGetResponse (buffer, sizeof(buffer), 2))
 	{
-		fatalx(EXIT_FAILURE, "Serial timeout with ONEAC UPS on %s\n",
-																device_path);
+		fatalx(EXIT_FAILURE, "Serial timeout with ONEAC UPS on %s\n", device_path);
 	}
 	else
 	{
-		if (strncmp(buffer,FAMILY_ON,FAMILY_SIZE) != 0 &&
-			strncmp(buffer,FAMILY_OZ,FAMILY_SIZE) != 0 &&
-			strncmp(buffer,FAMILY_OB,FAMILY_SIZE) != 0 &&
-			strncmp(buffer,FAMILY_EG,FAMILY_SIZE) != 0)
+		if (strncmp(buffer, FAMILY_ON, FAMILY_SIZE) != 0 &&
+			strncmp(buffer, FAMILY_OZ, FAMILY_SIZE) != 0 &&
+			strncmp(buffer, FAMILY_OB, FAMILY_SIZE) != 0 &&
+			strncmp(buffer, FAMILY_EG, FAMILY_SIZE) != 0)
 		{
-			fatalx(EXIT_FAILURE, "Did not find an ONEAC UPS on %s\n",
-																device_path);
+			fatalx(EXIT_FAILURE, "Did not find an ONEAC UPS on %s\n", device_path);
 		}
 	}
 
@@ -256,12 +258,11 @@ void upsdrv_initinfo(void)
 	/* set some stuff that shouldn't change after initialization */
 	/* this stuff is common to all families of UPS */
 
-	ser_send(upsfd,"%c%s",GET_ALL,COMMAND_END);
+	ser_send(upsfd,"%c%s", GET_ALL, COMMAND_END);
 
 	if (strncmp(UpsFamily, FAMILY_EG, FAMILY_SIZE) == 0)
 	{
-		RetValue = OneacGetResponse (buffer, sizeof(buffer),
-														GETALL_EG_RESP_SIZE);
+		RetValue = OneacGetResponse (buffer, sizeof(buffer), GETALL_EG_RESP_SIZE);
 	}
 	else
 	{
@@ -270,8 +271,7 @@ void upsdrv_initinfo(void)
 
 	if(RetValue)
 	{
-		fatalx(EXIT_FAILURE, "Serial timeout(2) with ONEAC UPS on %s\n",
-																device_path);
+		fatalx(EXIT_FAILURE, "Serial timeout(2) with ONEAC UPS on %s\n", device_path);
 	}
 
 	/* Manufacturer */
@@ -286,9 +286,13 @@ void upsdrv_initinfo(void)
 
 	/* Shutdown delay in seconds...can be changed by user */
 	if (getval("offdelay") == NULL)
+	{
 		dstate_setinfo("ups.delay.shutdown", "0");
+	}
 	else
+	{
 		dstate_setinfo("ups.delay.shutdown", "%s", getval("offdelay"));
+	}
 
 	dstate_setflags("ups.delay.shutdown", ST_FLAG_STRING | ST_FLAG_RW);
 	dstate_setaux("ups.delay.shutdown", GET_SHUTDOWN_RESP_SIZE);
@@ -334,12 +338,11 @@ void upsdrv_initinfo(void)
 
 		dstate_setaux("ups.delay.shutdown", GETX_SHUTDOWN_RESP_SIZE);
 
-		ser_flush_in(upsfd,"",0);
-		ser_send(upsfd,"%c%s",GETX_ALL_2,COMMAND_END);
+		ser_flush_in(upsfd, "", 0);
+		ser_send(upsfd, "%c%s", GETX_ALL_2, COMMAND_END);
 		if(OneacGetResponse (buffer, sizeof(buffer), GETX_ALL2_RESP_SIZE))
 		{
-			fatalx(EXIT_FAILURE, "Serial timeout(3) with ONEAC UPS on %s\n",
-																device_path);
+			fatalx(EXIT_FAILURE, "Serial timeout(3) with ONEAC UPS on %s\n", device_path);
 		}
 
 		/* Low and high output trip points */
@@ -363,7 +366,7 @@ void upsdrv_initinfo(void)
 		strncpy(buffer2, buffer+82, 2);
 		buffer2[2]='\0';
 		timevalue = atoi(buffer2) * 60;		/* Change minutes to seconds */
-		dstate_setinfo("battery.runtime.low", "%d",timevalue);
+		dstate_setinfo("battery.runtime.low", "%d", timevalue);
 		dstate_setflags("battery.runtime.low", ST_FLAG_STRING | ST_FLAG_RW);
 		dstate_setaux("battery.runtime.low", 2);
 
@@ -393,8 +396,7 @@ void upsdrv_initinfo(void)
 
 		/* Serial number */
 		dstate_setinfo("device.serial", "%.4s-%.4s", buffer+18, buffer+22);
-		printf("Found %.10s UPS with serial number %.4s-%.4s\n",
-												buffer2, buffer+18, buffer+22);
+		printf("Found %.10s UPS with serial number %.4s-%.4s\n", buffer2, buffer+18, buffer+22);
 
 		/* Manufacture Date */
 		dstate_setinfo("ups.mfr.date", "%.6s (yymmdd)", buffer+38);
@@ -417,14 +419,12 @@ void upsdrv_initinfo(void)
 		if ((strncmp (UpsFamily, FAMILY_OB, FAMILY_SIZE) == 0) ||
 			(strcmp (dstate_getinfo("ups.firmware"), MIN_ALLOW_FW) >= 0 ))
 		{
-			upsdebugx (2,"Can get output window min/max! (%s)",
-												dstate_getinfo("ups.firmware"));
+			upsdebugx (2,"Can get output window min/max! (%s)", dstate_getinfo("ups.firmware"));
 
-			ser_send(upsfd,"%s%s",GETX_ALLOW_RANGE,COMMAND_END);
+			ser_send(upsfd, "%s%s", GETX_ALLOW_RANGE, COMMAND_END);
 			if(OneacGetResponse (buffer, sizeof(buffer), GETX_RANGE_RESP_SIZE))
 			{
-				fatalx(EXIT_FAILURE,
-						"Serial timeout(4) with ONEAC UPS on %s\n",device_path);
+				fatalx(EXIT_FAILURE, "Serial timeout(4) with ONEAC UPS on %s\n",device_path);
 			}
 
 			strncpy(buffer2, buffer, 3);
@@ -459,8 +459,7 @@ void upsdrv_initinfo(void)
 		else
 		{
 			/* Use default values from firmware */
-			upsdebugx (2,"Using trip defaults (%s)...",
-												dstate_getinfo("ups.firmware"));
+			upsdebugx (2, "Using trip defaults (%s)...", dstate_getinfo("ups.firmware"));
 
 			switch (VRange)				/* Held from initial use */
 			{
@@ -496,12 +495,12 @@ void upsdrv_updateinfo(void)
 	int timevalue;
 
 	/* Start with EG/ON information */
-	ser_flush_in(upsfd,"",0);  /*just in case*/
-	ser_send (upsfd,"%c%s", GET_ALL, COMMAND_END);
+	ser_flush_in(upsfd, "", 0);  /*just in case*/
+	ser_send (upsfd, "%c%s", GET_ALL, COMMAND_END);
 
 	if (strncmp(UpsFamily, FAMILY_EG, FAMILY_SIZE) == 0)
 	{
-		RetValue = OneacGetResponse (buffer,sizeof(buffer),GETALL_EG_RESP_SIZE);
+		RetValue = OneacGetResponse (buffer, sizeof(buffer), GETALL_EG_RESP_SIZE);
 	}
 	else
 	{
@@ -510,14 +509,13 @@ void upsdrv_updateinfo(void)
 
 	if ((RetValue != 0) && (CommTry == 0))
 	{
-		ser_comm_fail("Oneac UPS Comm failure continues on port %s",
-																device_path);
+		ser_comm_fail("Oneac UPS Comm failure continues on port %s", device_path);
 	}
 	else if (RetValue != 0)
 	{
 		if (--CommTry == 0)
 		{
-			ser_comm_fail("Oneac UPS Comm failure on port %s",device_path);
+			ser_comm_fail("Oneac UPS Comm failure on port %s", device_path);
 			dstate_datastale();
 		}
 		upsdebugx(2,"Oneac: Update serial comm retry value: %d", CommTry);
@@ -550,7 +548,9 @@ void upsdrv_updateinfo(void)
 			}
 
 			if (s & 0x02)			/* Low Battery */
+			{
 				status_set("LB");
+			}
 
 			if (s & 0x04)			/* General fault */
 			{
@@ -562,12 +562,16 @@ void upsdrv_updateinfo(void)
 			}
 
 			if (s & 0x08)			/* Replace Battery */
+			{
 				status_set("RB");
+			}
 
 /*			if (s & 0x10)	*/		/* High Line */
 
 			if (s & 0x20)			/* Unit is hot */
+			{
 				alarm_set("OVERHEAT");
+			}
 		}
 
 		/*take care of the reason why the UPS last transferred to battery*/
@@ -576,49 +580,51 @@ void upsdrv_updateinfo(void)
 				dstate_setinfo("input.transfer.reason",	"Blackout");
 				break;
 			case XFER_LOW_VOLT :
-				dstate_setinfo("input.transfer.reason",
-					"Low Input Voltage");
+				dstate_setinfo("input.transfer.reason", "Low Input Voltage");
 				break;
 			case XFER_HI_VOLT :
-				dstate_setinfo("input.transfer.reason",
-					"High Input Voltage");
+				dstate_setinfo("input.transfer.reason", "High Input Voltage");
 				break;
 			case NO_VALUE_YET :
-				dstate_setinfo("input.transfer.reason",
-					"No transfer yet.");
+				dstate_setinfo("input.transfer.reason", "No transfer yet.");
 				break;
 			default :
-				upslogx(LOG_INFO,"Oneac: Unknown reason for UPS battery"
-										" transfer [%c]", buffer[13]);
+				upslogx(LOG_INFO,"Oneac: Unknown reason for UPS battery transfer [%c]", buffer[13]);
 		}
 
 		/* now update info for only the non-EG families of UPS*/
 
 		if (strncmp(UpsFamily, FAMILY_EG, FAMILY_SIZE) != 0)
 		{
-			dstate_setinfo("ups.load", "0%.2s",buffer+31);
+			dstate_setinfo("ups.load", "0%.2s", buffer+31);
 
 			/* Output ON or OFF? */
 			if(buffer[27] == NO_VALUE_YET)
+			{
 				status_set("OFF");
+			}
 
 			/*battery charge*/
 			if(buffer[10] == YES)
-				dstate_setinfo("battery.charge", "0%.2s",buffer+33);
+			{
+				dstate_setinfo("battery.charge", "0%.2s", buffer+33);
+			}
 			else
+			{
 				dstate_setinfo("battery.charge", "100");
+			}
 
 			EliminateLeadingZeroes (buffer+35, 3, buffer2, sizeof(buffer2));
-			dstate_setinfo("input.voltage", "%s",buffer2);
+			dstate_setinfo("input.voltage", "%s", buffer2);
 
 			EliminateLeadingZeroes (buffer+38, 3, buffer2, sizeof(buffer2));
-			dstate_setinfo("input.voltage.minimum", "%s",buffer2);
+			dstate_setinfo("input.voltage.minimum", "%s", buffer2);
 
 			EliminateLeadingZeroes (buffer+41, 3, buffer2, sizeof(buffer2));
-			dstate_setinfo("input.voltage.maximum", "%s",buffer2);
+			dstate_setinfo("input.voltage.maximum", "%s", buffer2);
 
 			EliminateLeadingZeroes (buffer+44, 3, buffer2, sizeof(buffer2));
-			dstate_setinfo("output.voltage", "%s",buffer2);
+			dstate_setinfo("output.voltage", "%s", buffer2);
 
 			if (buffer[15] == NO_VALUE_YET)
 			{
@@ -631,8 +637,7 @@ void upsdrv_updateinfo(void)
 
 				if(buffer[15] != HIGH_COUNT)
 				{
-					EliminateLeadingZeroes (buffer+15, 3, buffer2,
-															sizeof(buffer2));
+					EliminateLeadingZeroes (buffer+15, 3, buffer2, sizeof(buffer2));
 					dstate_setinfo("ups.timer.shutdown", "%s", buffer2);
 				}
 				else
@@ -642,7 +647,9 @@ void upsdrv_updateinfo(void)
 			}
 
 			if (buffer[47] == YES)
+			{
 				status_set("BOOST");
+			}
 		}
 
 		/* Now update info for only the OZ/OB families of UPS */
@@ -651,16 +658,14 @@ void upsdrv_updateinfo(void)
 			(strncmp(UpsFamily, FAMILY_OB, FAMILY_SIZE) == 0))
 		{
 			ser_flush_in(upsfd,"",0);  /*just in case*/
-			ser_send (upsfd,"%c%s",GETX_ALL_1,COMMAND_END);
-			RetValue = OneacGetResponse (buffer, sizeof(buffer),
-														GETX_ALL1_RESP_SIZE);
+			ser_send (upsfd, "%c%s", GETX_ALL_1, COMMAND_END);
+			RetValue = OneacGetResponse (buffer, sizeof(buffer), GETX_ALL1_RESP_SIZE);
 
 			if(RetValue)
 			{
 				if (--CommTry == 0)
 				{
-					ser_comm_fail("Oneac (OZ) UPS Comm failure on port %s",
-																device_path);
+					ser_comm_fail("Oneac (OZ) UPS Comm failure on port %s", device_path);
 					dstate_datastale();
 				}
 
@@ -672,53 +677,50 @@ void upsdrv_updateinfo(void)
 				CommTry = COMM_TRIES;		/* Reset count */
 
 				EliminateLeadingZeroes (buffer+57, 5, buffer2, sizeof(buffer2));
-				dstate_setinfo("ups.realpower", "%s",buffer2);
+				dstate_setinfo("ups.realpower", "%s", buffer2);
 
-				dstate_setinfo("input.frequency", "%.2s.%c",
-														buffer+42,buffer[44]);
-				dstate_setinfo("output.frequency", "%.2s.%c",
-														buffer+76, buffer[78]);
+				dstate_setinfo("input.frequency", "%.2s.%c", buffer+42, buffer[44]);
+				dstate_setinfo("output.frequency", "%.2s.%c", buffer+76, buffer[78]);
 
 				EliminateLeadingZeroes (buffer+29, 3, buffer2, sizeof(buffer2));
-				dstate_setinfo("battery.voltage", "%s.%c",buffer2, buffer[32]);
+				dstate_setinfo("battery.voltage", "%s.%c", buffer2, buffer[32]);
 
-				dstate_setinfo("ups.temperature", "%.2s",buffer+13);
-				dstate_setinfo("ups.load", "%.3s",buffer+73);
+				dstate_setinfo("ups.temperature", "%.2s", buffer+13);
+				dstate_setinfo("ups.load", "%.3s", buffer+73);
 
 				strncpy(buffer2, buffer+19, 4);
 				buffer2[4]='\0';
 				timevalue = atoi(buffer2) * 60;		/* Change mins to secs */
-				dstate_setinfo("battery.runtime", "%d",timevalue);
+				dstate_setinfo("battery.runtime", "%d", timevalue);
 
 				/* Now some individual requests... */
 
 				/* Battery replace date */
-				ser_send (upsfd,"%c%s",GETX_BATT_REPLACED,COMMAND_END);
-				if(!OneacGetResponse (buffer, sizeof(buffer),
-														GETX_DATE_RESP_SIZE))
+				ser_send (upsfd, "%c%s", GETX_BATT_REPLACED, COMMAND_END);
+				if(!OneacGetResponse (buffer, sizeof(buffer), GETX_DATE_RESP_SIZE))
+				{
 					dstate_setinfo("battery.date", "%.6s (yymmdd)", buffer);
+				}
 
 				/* Low and high output trip points */
-				ser_send (upsfd,"%c%s",GETX_LOW_OUT_ALLOW,COMMAND_END);
-				if(!OneacGetResponse (buffer, sizeof(buffer),
-														GETX_ALLOW_RESP_SIZE))
+				ser_send (upsfd, "%c%s", GETX_LOW_OUT_ALLOW, COMMAND_END);
+				if(!OneacGetResponse (buffer, sizeof(buffer), GETX_ALLOW_RESP_SIZE))
 				{
-					EliminateLeadingZeroes (buffer, 3, buffer2,sizeof(buffer2));
+					EliminateLeadingZeroes (buffer, 3, buffer2, sizeof(buffer2));
 					dstate_setinfo("input.transfer.low", "%s", buffer2);
 				}
 
-				ser_send (upsfd,"%c%s",GETX_HI_OUT_ALLOW,COMMAND_END);
-				if(!OneacGetResponse (buffer, sizeof(buffer),
-														GETX_ALLOW_RESP_SIZE))
+				ser_send (upsfd, "%c%s", GETX_HI_OUT_ALLOW, COMMAND_END);
+				if(!OneacGetResponse (buffer, sizeof(buffer), GETX_ALLOW_RESP_SIZE))
+				{
 					dstate_setinfo("input.transfer.high", "%s", buffer);
+				}
 
 				/* Restart delay */
-				ser_send (upsfd,"%c%s",GETX_RESTART_DLY,COMMAND_END);
-				if(!OneacGetResponse (buffer, sizeof(buffer),
-														GETX_RSTRT_RESP_SIZE))
+				ser_send (upsfd, "%c%s", GETX_RESTART_DLY, COMMAND_END);
+				if(!OneacGetResponse (buffer, sizeof(buffer), GETX_RSTRT_RESP_SIZE))
 				{
-					EliminateLeadingZeroes (buffer, 4, buffer2,
-															sizeof(buffer2));
+					EliminateLeadingZeroes (buffer, 4, buffer2, sizeof(buffer2));
 					dstate_setinfo("ups.delay.start", "%s", buffer2);
 				}
 
@@ -743,45 +745,46 @@ void upsdrv_updateinfo(void)
 				}
 
 				/* Auto start setting */
-				ser_send (upsfd,"%s%s",GETX_AUTO_START,COMMAND_END);
+				ser_send (upsfd, "%s%s", GETX_AUTO_START, COMMAND_END);
 				if(!OneacGetResponse (buffer, sizeof(buffer), 1))
 				{
 					if (buffer[0] == '0')
+					{
 						dstate_setinfo("ups.start.auto", "yes");
+					}
 					else
+					{
 						dstate_setinfo("ups.start.auto", "no");
+					}
 				}
 
 				/* Low Batt at time */
-				ser_send (upsfd,"%c%s",GETX_LOW_BATT_TIME,COMMAND_END);
+				ser_send (upsfd, "%c%s", GETX_LOW_BATT_TIME, COMMAND_END);
 				if(!OneacGetResponse (buffer, sizeof(buffer), 2))
 				{
 					strncpy(buffer2, buffer, 2);
 					buffer2[2]='\0';
 					timevalue = atoi(buffer2) * 60;		/* Mins to secs */
-					dstate_setinfo("battery.runtime.low", "%d",timevalue);
+					dstate_setinfo("battery.runtime.low", "%d", timevalue);
 				}
 
 				/* Shutdown timer */
-				ser_send (upsfd,"%c%s",GETX_SHUTDOWN,COMMAND_END);
-				if(!OneacGetResponse (buffer, sizeof(buffer),
-													GETX_SHUTDOWN_RESP_SIZE))
+				ser_send (upsfd, "%c%s", GETX_SHUTDOWN, COMMAND_END);
+				if(!OneacGetResponse (buffer, sizeof(buffer), GETX_SHUTDOWN_RESP_SIZE))
 				{
 					/* ON would have handled NO_VALUE_YET and setting FSD
 					 *  above so only deal with counter value here.
 					 */
 					if (buffer[0] != NO_VALUE_YET)
 					{
-						EliminateLeadingZeroes (buffer, 5, buffer2,
-															sizeof(buffer2));
+						EliminateLeadingZeroes (buffer, 5, buffer2, sizeof(buffer2));
 						dstate_setinfo("ups.timer.shutdown", "%s", buffer2);
 					}
 				}
 
 				/* Restart timer */
-				ser_send (upsfd,"%s%s",GETX_RESTART_COUNT,COMMAND_END);
-				if(!OneacGetResponse (buffer, sizeof(buffer),
-														GETX_RSTRT_RESP_SIZE))
+				ser_send (upsfd, "%s%s", GETX_RESTART_COUNT, COMMAND_END);
+				if(!OneacGetResponse (buffer, sizeof(buffer), GETX_RSTRT_RESP_SIZE))
 				{
 					if (atoi(buffer) == 0)
 					{
@@ -789,8 +792,7 @@ void upsdrv_updateinfo(void)
 					}
 					else
 					{
-						EliminateLeadingZeroes (buffer, 4, buffer2,
-															sizeof(buffer2));
+						EliminateLeadingZeroes (buffer, 4, buffer2, sizeof(buffer2));
 						dstate_setinfo("ups.timer.start", "%s", buffer2);
 					}
 				}
@@ -835,6 +837,11 @@ void upsdrv_help(void)
 	printf("You must set the UPS interface card DIP switch to 9600 BPS\n");
 }
 
+/* optionally tweak prognames[] entries */
+void upsdrv_tweak_prognames(void)
+{
+}
+
 void upsdrv_cleanup(void)
 {
 	ser_close(upsfd, device_path);
@@ -842,25 +849,29 @@ void upsdrv_cleanup(void)
 
 void upsdrv_makevartable(void)
 {
-	addvar(VAR_VALUE,"testtime",
-		"Change battery test time from the 2 minute default.");
+	addvar(VAR_VALUE, "testtime", "Change battery test time from the 2 minute default.");
 
-	addvar(VAR_VALUE,"offdelay",
-		"Change shutdown delay time from 0 second default.");
+	addvar(VAR_VALUE,"offdelay", "Change shutdown delay time from 0 second default.");
 }
 
 int instcmd(const char *cmdname, const char *extra)
 {
 	int i;
 
-	upsdebugx(2, "In instcmd with %s and extra %s.", cmdname, extra);
+	/* May be used in logging below, but not as a command argument */
+	NUT_UNUSED_VARIABLE(extra);
+	upsdebug_INSTCMD_STARTING(cmdname, extra);
 
-	if (!strcasecmp(cmdname, "test.failure.start")) {
-		ser_send(upsfd,"%s%s",SIM_PWR_FAIL,COMMAND_END);
+	if (!strcasecmp(cmdname, "test.failure.start"))
+	{
+		upslog_INSTCMD_POWERSTATE_MAYBE(cmdname, extra);
+		ser_send(upsfd, "%s%s", SIM_PWR_FAIL, COMMAND_END);
 		return STAT_INSTCMD_HANDLED;
 	}
 
-	if (!strcasecmp(cmdname, "shutdown.return")) {
+	if (!strcasecmp(cmdname, "shutdown.return"))
+	{
+		upslog_INSTCMD_POWERSTATE_CHANGE(cmdname, extra);
 
 		i = atoi(dstate_getinfo("ups.delay.shutdown"));
 
@@ -868,84 +879,97 @@ int instcmd(const char *cmdname, const char *extra)
 			(strncmp (UpsFamily, FAMILY_OB, FAMILY_SIZE) == 0))
 		{
 			upsdebugx(3, "Shutdown using %c%d...", DELAYED_SHUTDOWN_PREFIX, i);
-			ser_send(upsfd,"%c%d%s",DELAYED_SHUTDOWN_PREFIX, i, COMMAND_END);
+			ser_send(upsfd, "%c%d%s", DELAYED_SHUTDOWN_PREFIX, i, COMMAND_END);
 		}
 		else
 		{
-			upsdebugx(3, "Shutdown using %c%03d...",DELAYED_SHUTDOWN_PREFIX, i);
-			ser_send(upsfd,"%c%03d%s",DELAYED_SHUTDOWN_PREFIX, i, COMMAND_END);
+			upsdebugx(3, "Shutdown using %c%03d...", DELAYED_SHUTDOWN_PREFIX, i);
+			ser_send(upsfd, "%c%03d%s", DELAYED_SHUTDOWN_PREFIX, i, COMMAND_END);
 		}
 
 		return STAT_INSTCMD_HANDLED;
 	}
 
-	if(!strcasecmp(cmdname, "shutdown.reboot")) {
+	if (!strcasecmp(cmdname, "shutdown.reboot"))
+	{
+		upslog_INSTCMD_POWERSTATE_CHANGE(cmdname, extra);
 		ser_send(upsfd, "%s", SHUTDOWN);
 		return STAT_INSTCMD_HANDLED;
 	}
 
-	if (!strcasecmp(cmdname, "shutdown.stop")) {
-		ser_send(upsfd,"%c%s",DELAYED_SHUTDOWN_PREFIX,COMMAND_END);
+	if (!strcasecmp(cmdname, "shutdown.stop"))
+	{
+		upslog_INSTCMD_POWERSTATE_MAYBE(cmdname, extra);
+		ser_send(upsfd, "%c%s", DELAYED_SHUTDOWN_PREFIX, COMMAND_END);
 		return STAT_INSTCMD_HANDLED;
 	}
 
-	if (!strcasecmp(cmdname, "test.battery.start.quick")) {
+	if (!strcasecmp(cmdname, "test.battery.start.quick"))
+	{
+		upslog_INSTCMD_POWERSTATE_MAYBE(cmdname, extra);
 		do_battery_test();
 		return STAT_INSTCMD_HANDLED;
 	}
 
 	if (!strcasecmp(cmdname, "test.battery.start.deep")) {
+		upslog_INSTCMD_POWERSTATE_MAYBE(cmdname, extra);
 		ser_send(upsfd, "%s%s", TEST_BATT_DEEP, COMMAND_END);
 		return STAT_INSTCMD_HANDLED;
 	}
 
 	if (!strcasecmp(cmdname, "test.battery.stop"))
 	{
+		upslog_INSTCMD_POWERSTATE_MAYBE(cmdname, extra);
 		if ((strncmp (UpsFamily, FAMILY_EG, FAMILY_SIZE) == 0) ||
 			(strncmp (UpsFamily, FAMILY_ON, FAMILY_SIZE) == 0))
 		{
-			ser_send(upsfd,"%s00%s",BAT_TEST_PREFIX,COMMAND_END);
+			ser_send(upsfd, "%s00%s", BAT_TEST_PREFIX, COMMAND_END);
 		}
 		else
 		{
-			ser_send(upsfd,"%c%s",TEST_ABORT,COMMAND_END);
+			ser_send(upsfd, "%c%s", TEST_ABORT, COMMAND_END);
 		}
 		return STAT_INSTCMD_HANDLED;
 	}
 
-	if (!strcasecmp(cmdname, "reset.input.minmax")) {
-		ser_send(upsfd,"%c%s",RESET_MIN_MAX, COMMAND_END);
+	if (!strcasecmp(cmdname, "reset.input.minmax"))
+	{
+		ser_send(upsfd, "%c%s", RESET_MIN_MAX, COMMAND_END);
 		return STAT_INSTCMD_HANDLED;
 	}
 
-	if (!strcasecmp(cmdname, "beeper.enable")) {
-		ser_send(upsfd,"%c%c%s",SETX_BUZZER_PREFIX, BUZZER_ENABLED,COMMAND_END);
+	if (!strcasecmp(cmdname, "beeper.enable"))
+	{
+		ser_send(upsfd, "%c%c%s", SETX_BUZZER_PREFIX, BUZZER_ENABLED, COMMAND_END);
 		return STAT_INSTCMD_HANDLED;
 	}
 
-	if (!strcasecmp(cmdname, "beeper.disable")) {
-		ser_send(upsfd,"%c%c%s",SETX_BUZZER_PREFIX,BUZZER_DISABLED,COMMAND_END);
+	if (!strcasecmp(cmdname, "beeper.disable"))
+	{
+		ser_send(upsfd, "%c%c%s", SETX_BUZZER_PREFIX, BUZZER_DISABLED, COMMAND_END);
 		return STAT_INSTCMD_HANDLED;
 	}
 
-	if (!strcasecmp(cmdname, "beeper.mute")) {
-		ser_send(upsfd,"%c%c%s",SETX_BUZZER_PREFIX, BUZZER_MUTED, COMMAND_END);
+	if (!strcasecmp(cmdname, "beeper.mute"))
+	{
+		ser_send(upsfd,"%c%c%s", SETX_BUZZER_PREFIX, BUZZER_MUTED, COMMAND_END);
 		return STAT_INSTCMD_HANDLED;
 	}
 
-	if (!strcasecmp(cmdname, "test.panel.start")) {
-		ser_send(upsfd,"%s%s",TEST_INDICATORS, COMMAND_END);
+	if (!strcasecmp(cmdname, "test.panel.start"))
+	{
+		ser_send(upsfd,"%s%s", TEST_INDICATORS, COMMAND_END);
 		return STAT_INSTCMD_HANDLED;
 	}
 
-	upslogx(LOG_NOTICE, "instcmd: unknown command [%s]", cmdname);
+	upslog_INSTCMD_UNKNOWN(cmdname, extra);
 	return STAT_INSTCMD_UNKNOWN;
 }
 
 
 int setcmd(const char* varname, const char* setvalue)
 {
-	upsdebugx(2, "In setcmd for %s with %s...", varname, setvalue);
+	upsdebug_SET_STARTING(varname, setvalue);
 
 	if (!strcasecmp(varname, "ups.delay.shutdown"))
 	{
@@ -954,6 +978,7 @@ int setcmd(const char* varname, const char* setvalue)
 		{
 			if (atoi(setvalue) > 65535)
 			{
+				/* FIXME: ..._INVALID? ..._CONVERSION_FAILED? */
 				upsdebugx(2, "Too big for OZ/OB (>65535)...(%s)", setvalue);
 				return STAT_SET_UNKNOWN;
 			}
@@ -962,6 +987,7 @@ int setcmd(const char* varname, const char* setvalue)
 		{
 			if (atoi(setvalue) > 999)
 			{
+				/* FIXME: ..._INVALID? ..._CONVERSION_FAILED? */
 				upsdebugx(2, "Too big for EG/ON (>999)...(%s)", setvalue);
 				return STAT_SET_UNKNOWN;
 			}
@@ -1015,7 +1041,7 @@ int setcmd(const char* varname, const char* setvalue)
 	{
 		if (atoi(setvalue) <= 9999)
 		{
-			ser_send(upsfd,"%s%s%s",SETX_RESTART_DELAY, setvalue, COMMAND_END);
+			ser_send(upsfd,"%s%s%s", SETX_RESTART_DELAY, setvalue, COMMAND_END);
 
 			dstate_setinfo("ups.delay.start", "%s", setvalue);
 			return STAT_SET_HANDLED;
@@ -1030,7 +1056,7 @@ int setcmd(const char* varname, const char* setvalue)
 	{
 		if (atoi(setvalue) <= 99)
 		{
-			ser_send(upsfd,"%s%s%s",SETX_LOWBATT_AT, setvalue, COMMAND_END);
+			ser_send(upsfd,"%s%s%s", SETX_LOWBATT_AT, setvalue, COMMAND_END);
 
 			dstate_setinfo("battery.runtime.low", "%s", setvalue);
 			return STAT_SET_HANDLED;
@@ -1045,13 +1071,13 @@ int setcmd(const char* varname, const char* setvalue)
 	{
 		if (!strcasecmp(setvalue, "yes"))
 		{
-			ser_send(upsfd,"%c0%s",SETX_AUTO_START, COMMAND_END);
+			ser_send(upsfd,"%c0%s", SETX_AUTO_START, COMMAND_END);
 			dstate_setinfo("ups.start.auto", "yes");
 			return STAT_SET_HANDLED;
 		}
 		else if (!strcasecmp(setvalue, "no"))
 		{
-			ser_send(upsfd,"%c1%s",SETX_AUTO_START, COMMAND_END);
+			ser_send(upsfd,"%c1%s", SETX_AUTO_START, COMMAND_END);
 			dstate_setinfo("ups.start.auto", "no");
 			return STAT_SET_HANDLED;
 		}
@@ -1059,7 +1085,6 @@ int setcmd(const char* varname, const char* setvalue)
 		return STAT_SET_UNKNOWN;
 	}
 
-	upslogx(LOG_NOTICE, "setcmd: unknown command [%s]", varname);
-
+	upslog_SET_UNKNOWN(varname, setvalue);
 	return STAT_SET_UNKNOWN;
 }
