@@ -23,7 +23,7 @@
 #include "nut_stdint.h"
 
 #define DRIVER_NAME                         "PiJuice UPS driver"
-#define DRIVER_VERSION                      "0.13"
+#define DRIVER_VERSION                      "0.16"
 
 /*
  * Linux I2C userland is a bit of a mess until distros refresh to
@@ -340,12 +340,9 @@ static void get_charge_level_hi_res(void)
 static void get_status(void)
 {
 	uint8_t	cmd = STATUS_CMD, data, batteryStatus, powerInput, powerInput5vIo;
-	char	status_buf[ST_MAX_VALUE_LEN];
 
 	upsdebugx( 3, __func__ );
 
-	memset( status_buf, 0, ST_MAX_VALUE_LEN );
-
 	I2C_READ_BYTE( upsfd, cmd, __func__ )
 
 	batteryStatus = data >> 2 & 0x03;
@@ -422,17 +419,17 @@ static void get_status(void)
 		if ( battery_charge_level <= LOW_BATTERY_THRESHOLD )
 		{
 			upsdebugx( 1, "Battery Charge Status: LOW" );
-			snprintfcat( status_buf, ST_MAX_VALUE_LEN, "LB " );
+			status_set("LB");
 		}
 		else if ( battery_charge_level > HIGH_BATTERY_THRESHOLD )
 		{
 			upsdebugx( 1, "Battery Charge Status: HIGH" );
-			snprintfcat( status_buf, ST_MAX_VALUE_LEN, "HB " );
+			status_set("HB");
 		}
 	}
 	else if ( batteryStatus == BATT_NOT_PRESENT )
 	{
-		snprintfcat( status_buf, ST_MAX_VALUE_LEN, "RB " );
+		status_set("RB");
 	}
 
 	if ( batteryStatus  <= BATT_NOT_PRESENT &&
@@ -451,10 +448,10 @@ static void get_status(void)
 			battery_power = 0;
 			upsdebugx( 1, "On USB power [%d:%d:%d]", usb_power, gpio_power, battery_power );
 
-			snprintfcat( status_buf, sizeof(status_buf), "OL" );
+			status_set("OL");
 			if ( batteryStatus == BATT_CHARGING_FROM_5V )
 			{
-				snprintfcat( status_buf, sizeof( status_buf ), " CHRG" );
+				status_set("CHRG");
 				upsdebugx( 1, "Battery Charger Status: charging" );
 				dstate_setinfo( "battery.charger.status", "%s", "charging" );
 			}
@@ -463,7 +460,6 @@ static void get_status(void)
 				upsdebugx( 1, "Battery Charger Status: resting" );
 				dstate_setinfo( "battery.charger.status", "%s", "resting" );
 			}
-			status_set( status_buf );
 		}
 		else if ( powerInput5vIo == POWER_NOT_PRESENT &&
 		      ( powerInput   != POWER_NOT_PRESENT &&
@@ -478,17 +474,15 @@ static void get_status(void)
 			battery_power = 0;
 			upsdebugx( 1, "On 5V_GPIO power [%d:%d:%d]", usb_power, gpio_power, battery_power );
 
-			snprintfcat( status_buf, sizeof(status_buf), "OL" );
+			status_set("OL");
 			if ( batteryStatus == BATT_CHARGING_FROM_IN )
 			{
-				snprintfcat( status_buf, sizeof(status_buf), " CHRG" );
-				status_set( status_buf );
+				status_set("CHRG");
 				upsdebugx( 1, "Battery Charger Status: charging" );
 				dstate_setinfo( "battery.charger.status", "%s", "charging" );
 			}
 			else if ( batteryStatus == BATT_NORMAL )
 			{
-				status_set( status_buf );
 				upsdebugx( 1, "Battery Charger Status: resting" );
 				dstate_setinfo( "battery.charger.status", "%s", "resting" );
 			}
@@ -505,17 +499,15 @@ static void get_status(void)
 			battery_power = 0;
 			upsdebugx( 1, "On USB and 5V_GPIO power [%d:%d:%d]", usb_power, gpio_power, battery_power );
 
-			snprintfcat( status_buf, sizeof( status_buf ), "OL" );
+			status_set("OL");
 			if ( batteryStatus == BATT_CHARGING_FROM_IN )
 			{
-				snprintfcat( status_buf, sizeof(status_buf), " CHRG");
-				status_set( status_buf );
+				status_set("CHRG");
 				upsdebugx( 1, "Battery Charger Status: charging" );
 				dstate_setinfo("battery.charger.status", "%s", "charging");
 			}
 			else if ( batteryStatus == BATT_NORMAL )
 			{
-				status_set( status_buf );
 				upsdebugx( 1, "Battery Charger Status: resting" );
 				dstate_setinfo( "battery.charger.status", "%s", "resting" );
 			}
@@ -531,8 +523,8 @@ static void get_status(void)
 			battery_power = 1;
 			upsdebugx( 1, "On Battery power [%d:%d:%d]", usb_power, gpio_power, battery_power );
 
-			snprintfcat( status_buf, sizeof(status_buf), "OB DISCHRG" );
-			status_set( status_buf );
+			status_set("OB");
+			status_set("DISCHRG");
 		}
 	}
 }
@@ -833,19 +825,23 @@ void upsdrv_updateinfo(void)
 static
 int instcmd(const char *cmdname, const char *extra)
 {
+	/* May be used in logging below, but not as a command argument */
 	NUT_UNUSED_VARIABLE(extra);
+	upsdebug_INSTCMD_STARTING(cmdname, extra);
 
 	/* FIXME: Which one is this - "load.off",
 	 * "shutdown.stayoff" or "shutdown.return"? */
 
 	/* Shutdown UPS */
 	if (!strcasecmp(cmdname, "shutdown.stayoff")) {
+		upslog_INSTCMD_POWERSTATE_CHANGE(cmdname, extra);
+
 		set_power_off();
 
 		return STAT_INSTCMD_HANDLED;
 	}
 
-	upslogx(LOG_NOTICE, "instcmd: unknown command [%s] [%s]", cmdname, extra);
+	upslog_INSTCMD_UNKNOWN(cmdname, extra);
 	return STAT_INSTCMD_UNKNOWN;
 }
 
@@ -868,6 +864,11 @@ void upsdrv_help(void)
 	printf("\n");
 }
 
+/* optionally tweak prognames[] entries */
+void upsdrv_tweak_prognames(void)
+{
+}
+
 void upsdrv_makevartable(void)
 {
 	addvar(VAR_VALUE, "i2c_address", "Override i2c address setting");
