@@ -23,7 +23,7 @@
 #include <libpowerman.h>	/* pm_err_t and other beasts */
 
 #define DRIVER_NAME	"Powerman PDU client driver"
-#define DRIVER_VERSION	"0.15"
+#define DRIVER_VERSION	"0.17"
 
 /* driver description structure */
 upsdrv_info_t upsdrv_info = {
@@ -54,7 +54,9 @@ static int instcmd(const char *cmdname, const char *extra)
 	char *cmdindex = NULL;
 	char outletname[SMALLBUF];
 
-	upsdebugx(1, "entering instcmd (%s)", cmdname);
+	/* May be used in logging below, but not as a command argument */
+	NUT_UNUSED_VARIABLE(extra);
+	upsdebug_INSTCMD_STARTING(cmdname, extra);
 
 	/* only consider the end of the command */
 	if ( (cmdsuffix = strrchr(cmdname, '.')) == NULL )
@@ -74,23 +76,26 @@ static int instcmd(const char *cmdname, const char *extra)
 
 	/* Power on the outlet */
 	if (!strcasecmp(cmdsuffix, "on")) {
+		upslog_INSTCMD_POWERSTATE_MAYBE(cmdname, extra);
 		rv = pm_node_on(pm, outletname);
 		return (rv==PM_ESUCCESS)?STAT_INSTCMD_HANDLED:STAT_INSTCMD_INVALID;
 	}
 
 	/* Power off the outlet */
 	if (!strcasecmp(cmdsuffix, "off")) {
+		upslog_INSTCMD_POWERSTATE_CHANGE(cmdname, extra);
 		rv = pm_node_off(pm, outletname);
 		return (rv==PM_ESUCCESS)?STAT_INSTCMD_HANDLED:STAT_INSTCMD_INVALID;
 	}
 
 	/* Cycle the outlet */
 	if (!strcasecmp(cmdsuffix, "cycle")) {
+		upslog_INSTCMD_POWERSTATE_CHANGE(cmdname, extra);
 		rv = pm_node_cycle(pm, outletname);
 		return (rv==PM_ESUCCESS)?STAT_INSTCMD_HANDLED:STAT_INSTCMD_INVALID;
 	}
 
-	upslogx(LOG_NOTICE, "instcmd: unknown command [%s] [%s]", cmdname, extra);
+	upslog_INSTCMD_UNKNOWN(cmdname, extra);
 	return STAT_INSTCMD_UNKNOWN;
 }
 
@@ -99,7 +104,7 @@ void upsdrv_updateinfo(void)
 	pm_err_t rv = PM_ESUCCESS;
 
 	if ( (rv = query_all(pm, WALKMODE_UPDATE)) != PM_ESUCCESS) {
-		upslogx(2, "Error: %s (%i)\n", pm_strerror(rv, ebuf, sizeof(ebuf)), errno);
+		upsdebugx(2, "Error: %s (%i)\n", pm_strerror(rv, ebuf, sizeof(ebuf)), errno);
 		/* FIXME: try to reconnect?
 		 *	dstate_datastale();
 		 */
@@ -122,7 +127,7 @@ void upsdrv_initinfo(void)
 
 	/* Now walk the data tree */
 	if ( (rv = query_all(pm, WALKMODE_INIT)) != PM_ESUCCESS) {
-		upslogx(2, "Error: %s\n", pm_strerror(rv, ebuf, sizeof(ebuf)));
+		upsdebugx(2, "Error: %s\n", pm_strerror(rv, ebuf, sizeof(ebuf)));
 		/* FIXME: try to reconnect?
 		 *	dstate_datastale();
 		 */
@@ -163,12 +168,14 @@ void upsdrv_shutdown(void)
 /*
 static int setvar(const char *varname, const char *val)
 {
+	upsdebug_SET_STARTING(varname, val);
+ 
 	if (!strcasecmp(varname, "outlet.n.delay.*")) {
 		...
 		return STAT_SET_HANDLED;
 	}
 
-	upslogx(LOG_NOTICE, "setvar: unknown variable [%s]", varname);
+	upslog_SET_UNKNOWN(varname, val);
 	return STAT_SET_UNKNOWN;
 }
 */
@@ -177,6 +184,11 @@ void upsdrv_help(void)
 {
 }
 
+/* optionally tweak prognames[] entries */
+void upsdrv_tweak_prognames(void)
+{
+}
+
 /* list flags and values that you want to receive via -x */
 void upsdrv_makevartable(void)
 {
