@@ -30,8 +30,8 @@
 
 #define ENDCHAR '\r'
 
-#define DRIVER_NAME "SMS Brazil UPS driver"
-#define DRIVER_VERSION "1.02"
+#define DRIVER_NAME	"SMS Brazil UPS driver"
+#define DRIVER_VERSION	"1.04"
 
 #define QUERY_SIZE 7
 #define BUFFER_SIZE 18
@@ -254,8 +254,12 @@ static int get_ups_features(void) {
 static int sms_instcmd(const char *cmdname, const char *extra) {
     size_t length;
 
+    upsdebug_INSTCMD_STARTING(cmdname, extra);
+
     if (!strcasecmp(cmdname, "test.battery.start")) {
         long delay = extra ? strtol(extra, NULL, 10) : 10;
+
+        upslog_INSTCMD_POWERSTATE_MAYBE(cmdname, extra);
         length = sms_prepare_test_battery_nsec(&bufOut[0], delay);
 
         if (ser_send_buf(upsfd, bufOut, length) == 0) {
@@ -267,6 +271,7 @@ static int sms_instcmd(const char *cmdname, const char *extra) {
     }
 
     if (!strcasecmp(cmdname, "test.battery.start.quick")) {
+        upslog_INSTCMD_POWERSTATE_MAYBE(cmdname, extra);
         length = sms_prepare_test_battery_low(&bufOut[0]);
 
         if (ser_send_buf(upsfd, bufOut, length) == 0) {
@@ -279,6 +284,7 @@ static int sms_instcmd(const char *cmdname, const char *extra) {
     }
 
     if (!strcasecmp(cmdname, "test.battery.stop")) {
+        upslog_INSTCMD_POWERSTATE_MAYBE(cmdname, extra);
         length = sms_prepare_cancel_test(&bufOut[0]);
 
         if (ser_send_buf(upsfd, bufOut, length) == 0) {
@@ -303,6 +309,7 @@ static int sms_instcmd(const char *cmdname, const char *extra) {
     }
 
     if (!strcasecmp(cmdname, "shutdown.return")) {
+        upslog_INSTCMD_POWERSTATE_CHANGE(cmdname, extra);
         length = sms_prepare_shutdown_restore(&bufOut[0]);
 
         if (ser_send_buf(upsfd, bufOut, length) == 0) {
@@ -324,6 +331,8 @@ static int sms_instcmd(const char *cmdname, const char *extra) {
                 upsdebugx(3, "tried to set up extra shutdown.reboot delay ut it was out of range, keeping default");
             }
         }
+
+        upslog_INSTCMD_POWERSTATE_CHANGE(cmdname, extra);
         length = sms_prepare_shutdown_nsec(&bufOut[0], delay);
 
         if (ser_send_buf(upsfd, bufOut, length) == 0) {
@@ -336,6 +345,7 @@ static int sms_instcmd(const char *cmdname, const char *extra) {
     }
 
     if (!strcasecmp(cmdname, "shutdown.stop")) {
+        upslog_INSTCMD_POWERSTATE_MAYBE(cmdname, extra);
         length = sms_prepare_cancel_shutdown(&bufOut[0]);
 
         if (ser_send_buf(upsfd, bufOut, length) == 0) {
@@ -347,11 +357,13 @@ static int sms_instcmd(const char *cmdname, const char *extra) {
         return STAT_INSTCMD_HANDLED;
     }
 
-    upslogx(LOG_NOTICE, "sms_instcmd: unknown command [%s]", cmdname);
+    upslog_INSTCMD_UNKNOWN(cmdname, extra);
     return STAT_INSTCMD_UNKNOWN;
 }
 
 static int sms_setvar(const char *varname, const char *val) {
+    upsdebug_SET_STARTING(varname, val);
+
     if (!strcasecmp(varname, "ups.delay.reboot")) {
         int ipv = atoi(val);
         if (ipv >= 0)
@@ -359,6 +371,8 @@ static int sms_setvar(const char *varname, const char *val) {
         dstate_setinfo("ups.delay.reboot", "%u", bootdelay);
         return STAT_SET_HANDLED;
     }
+
+    upslog_SET_UNKNOWN(varname, val);
     return STAT_SET_UNKNOWN;
 }
 
@@ -558,6 +572,11 @@ void upsdrv_shutdown(void) {
 void upsdrv_help(void) {
 }
 
+/* optionally tweak prognames[] entries */
+void upsdrv_tweak_prognames(void)
+{
+}
+
 /* list flags and values that you want to receive via -x */
 void upsdrv_makevartable(void) {
     char msg[256];
