@@ -43,7 +43,7 @@
 #include "nut_float.h"
 
 #define DRIVER_NAME	"UPScode II UPS driver"
-#define DRIVER_VERSION	"0.93"
+#define DRIVER_VERSION	"0.95"
 
 /* driver description structure */
 upsdrv_info_t	upsdrv_info = {
@@ -460,6 +460,10 @@ void upsdrv_help(void)
 {
 }
 
+/* optionally tweak prognames[] entries */
+void upsdrv_tweak_prognames(void)
+{
+}
 
 void upsdrv_initups(void)
 {
@@ -907,12 +911,15 @@ static int instcmd (const char *auxcmd, const char *data)
 		return instcmd("beeper.enable", NULL);
 	}
 
-	upsdebugx(1, "Instcmd: %s %s", auxcmd, data ? data : "\"\"");
+	upsdebug_INSTCMD_STARTING(auxcmd, (data ? data : "\"\""));
 
 	for (cp = commands; cp->cmd; cp++) {
 		if (strcasecmp(cp->cmd, auxcmd)) {
 			continue;
 		}
+
+		upslog_INSTCMD_POWERSTATE_CHECKED(auxcmd, data);
+
 		upscsend(cp->upsc);
 		if (cp->upsp) {
 			upscsend(cp->upsp);
@@ -922,7 +929,7 @@ static int instcmd (const char *auxcmd, const char *data)
 		return STAT_INSTCMD_HANDLED;
 	}
 
-	upslogx(LOG_INFO, "instcmd: unknown command %s", auxcmd);
+	upslog_INSTCMD_UNKNOWN(auxcmd, data);
 	return STAT_INSTCMD_UNKNOWN;
 }
 
@@ -931,7 +938,7 @@ static int setvar (const char *var, const char *data)
 {
 	cmd_t *cp;
 
-	upsdebugx(1, "Setvar: %s %s", var, data);
+	upsdebug_SET_STARTING(var, data);
 
 	for (cp = variables; cp->cmd; cp++) {
 		if (strcasecmp(cp->cmd, var)) {
@@ -941,7 +948,8 @@ static int setvar (const char *var, const char *data)
 		return STAT_SET_HANDLED;
 	}
 
-	upslogx(LOG_INFO, "Setvar: unsettable variable %s", var);
+	upslogx(LOG_SET_UNKNOWN, "%s: unsettable variable %s",
+		__func__, NUT_STRARG(var));
 	return STAT_SET_UNKNOWN;
 }
 
