@@ -29,12 +29,13 @@
  */
 
 #define DRIVER_NAME	"Generic HID driver"
-#define DRIVER_VERSION	"0.62"
+#define DRIVER_VERSION	"0.69"
 
 #define HU_VAR_WAITBEFORERECONNECT "waitbeforereconnect"
 
 #include "main.h"	/* Must be first, includes "config.h" */
 #include "nut_stdint.h"
+#include "nut_float.h"
 #include "libhid.h"
 #include "usbhid-ups.h"
 #include "hidparser.h"
@@ -284,6 +285,7 @@ typedef struct {
 static status_lkp_t status_info[] = {
 	/* map internal status strings to bit masks */
 	{ "online", STATUS(ONLINE) },
+	{ "offline", STATUS(OFFLINE) },
 	{ "dischrg", STATUS(DISCHRG) },
 	{ "chrg", STATUS(CHRG) },
 	{ "lowbatt", STATUS(LOWBATT) },
@@ -339,7 +341,7 @@ static status_lkp_t status_info[] = {
 
 info_lkp_t online_info[] = {
 	{ 1, "online", NULL, NULL },
-	{ 0, "!online", NULL, NULL },
+	{ 0, "offline", NULL, NULL },	/* previously was "!online" but that proved ambiguous */
 	{ 0, NULL, NULL, NULL }
 };
 info_lkp_t discharging_info[] = {
@@ -620,16 +622,16 @@ info_lkp_t divide_by_10_conversion[] = {
    done with result! */
 static const char *divide_by_100_conversion_fun(double value)
 {
-       static char buf[20];
+	static char buf[20];
 
-       snprintf(buf, sizeof(buf), "%0.1f", value * 0.01);
+	snprintf(buf, sizeof(buf), "%0.1f", value * 0.01);
 
-       return buf;
+	return buf;
 }
 
 /* FIXME? Do we need an inverse "nuf()" here? */
 info_lkp_t divide_by_100_conversion[] = {
-       { 0, NULL, divide_by_100_conversion_fun, NULL }
+	{ 0, NULL, divide_by_100_conversion_fun, NULL }
 };
 
 /* returns statically allocated string - must not use it again before
@@ -658,6 +660,190 @@ info_lkp_t kelvin_celsius_conversion[] = {
 	{ 0, NULL, kelvin_celsius_conversion_fun, NULL }
 };
 
+static void analyze_mapping_usage(void) {
+	/* Check if the subdriver code (mappings) and the device report
+	 * sit together well. Note that for yet-unknown concepts, the
+	 * NUT driver developers can either raise a discussion on how
+	 * to best formalize that concept via docs/nut-names.txt, or
+	 * temporarily place them into "experimental.*" or "unmapped.*"
+	 * namespaces.
+	 *
+	 * Later also check that all defined mappings were used?
+	 * TBH, this is unlikely in practice, so of little value
+	 * (unless we are troubleshooting and under 5 or 10 data
+	 * points are served from actually the device, and not
+	 * from user configs or driver fallbacks).
+	 *
+	 * See also: similar methods in snmp-ups and nutdrv_qx.
+	 */
+	size_t	d, unused_count = 0, halfused_count = 0;
+	size_t	unused_bufsize = LARGEBUF, halfused_bufsize = LARGEBUF, unused_prevlen = 0, halfused_prevlen = 0, used_mappings = 0, known_mappings = 0;
+	int	ret_printf;
+	char	*unused_names = NULL, *halfused_names = NULL;
+	hid_info_t	*hidups_item;
+
+	/* FIXME? this activity is limited to when debugging is enabled, even
+	 *  if some of the messages below can be posted visibly at level 0.
+	 */
+	if (nut_debug_level < 1)
+		return;
+
+	upsdebugx(1, "%s: checking if the subdriver code (mappings) "
+		"consults all data points from the device report",
+		__func__);
+
+	if (!pDesc) {
+		upsdebugx(1, "%s: SKIP: pDesc==null", __func__);
+		return;
+	}
+
+	unused_names = xcalloc(unused_bufsize, sizeof(char));
+	halfused_names = xcalloc(halfused_bufsize, sizeof(char));
+
+	for (d = 0; d < pDesc->nitems; d++) {
+		HIDData_t	*pData = &pDesc->item[d];
+
+		if (pData && !(pData->mapping_handled)) {
+			char	*pName = HIDGetDataItem(pData, subdriver->utab);
+			const char	*pType = HIDDataType(pData);
+			int	retry = 0;
+			char	**pNames = &unused_names;
+			size_t	*pCount = &unused_count, *pPrevLen = &unused_prevlen, *pBufSize = &unused_bufsize;
+
+			if (!pName) {
+				upsdebugx(2, "%s: error getting a Report Path name, skipped", __func__);
+				continue;
+			} else {
+				/* check if this is a half-used name */
+				for (hidups_item = subdriver->hid2nut; hidups_item->info_type != NULL ; hidups_item++) {
+					/* Note: using a shortcut with mapping table and
+					 * its "hidpath" strings here, to avoid stringifying
+					 * all paths known from report descriptor - more so
+					 * in a nested loop. */
+					if (hidups_item->hidpath && hidups_item->hiddata
+					&& !strcasecmp(hidups_item->hidpath, pName)
+					&& hidups_item->hiddata->mapping_handled
+					) {
+						upsdebugx(5, "%s: Path '%s' is half-used: "
+							"Type '%s' was not touched via mapping, "
+							"but '%s' was used",
+							__func__, NUT_STRARG(pName),
+							NUT_STRARG(pType),
+							NUT_STRARG(HIDDataType(hidups_item->hiddata))
+							);
+						pNames = &halfused_names;
+						pCount = &halfused_count;
+						pPrevLen = &halfused_prevlen;
+						pBufSize = &halfused_bufsize;
+						break;
+					}
+				}
+			}
+
+			/* We may overflow the pre-allocated buffer,
+			 * so we loop here until snprintf() succeeds
+			 * or we are known to have failed completely.
+			 */
+			do {
+				retry = 0;
+				if (!*pNames) {
+					break;
+				}
+
+				upsdebugx(5, "%s: adding '%s (%s)' (%" PRIuSIZE " bytes) "
+					"to buffer of %" PRIuSIZE "/%" PRIuSIZE " bytes",
+					__func__, NUT_STRARG(pName), NUT_STRARG(pType),
+					pName ? strlen(pName) : 0,
+					*pPrevLen, *pBufSize);
+
+				ret_printf = snprintf(*pNames + *pPrevLen, *pBufSize - *pPrevLen - 1, "%s%s (%s)",
+					*pCount ? ", " : "", NUT_STRARG(pName), NUT_STRARG(pType));
+
+				upsdebugx(6, "%s: snprintf() returned %d", __func__, ret_printf);
+				(*pNames)[*pBufSize - 1] = '\0';
+
+				if (ret_printf < 0) {
+					upsdebugx(1, "%s: error collecting names, might not report unused descriptor names", __func__);
+				} else if ((size_t)ret_printf + *pPrevLen >= *pBufSize) {
+					if (*pBufSize < SIZE_MAX - LARGEBUF) {
+						*pBufSize = *pBufSize + LARGEBUF;
+						upsdebugx(1, "%s: buffer overflowed, trying to re-allocate as %" PRIuSIZE, __func__, *pBufSize);
+							*pNames = realloc(*pNames, *pBufSize);
+
+						if (!*pNames) {
+							upsdebugx(1, "%s: buffer overflowed, will not report unused descriptor names", __func__);
+						} else {
+							upsdebugx(5, "%s: buffer overflowed, but reallocated successfully - retrying", __func__);
+							/* Retry this loop */
+							retry = 1;
+						}
+					} else {
+						upsdebugx(1, "%s: buffer overflowed, might not report unused descriptor names", __func__);
+					}
+				} else {
+					*pPrevLen += (size_t)ret_printf;
+				}
+			} while (retry);
+
+			*pCount = *pCount + 1;
+		}
+	}
+
+	if (unused_count) {
+		upsdebugx(1, "%s: %" PRIuSIZE " items are present in the "
+			"report descriptor from HID UPS, but %" PRIuSIZE " "
+			"of them were completely not used by name via the "
+			"mapping defined in the selected NUT subdriver %s: %s",
+			__func__, pDesc->nitems, unused_count,
+			subdriver->name, NUT_STRARG(unused_names));
+	}
+
+	if (halfused_count) {
+		upsdebugx(1, "%s: %" PRIuSIZE " items are present in the "
+			"report descriptor from HID UPS, but %" PRIuSIZE " "
+			"of them have several Types named by same Path value, "
+			"where at least one of the names was used and other(s) "
+			"were not used by the mapping defined in "
+			"the selected NUT subdriver %s: %s",
+			__func__, pDesc->nitems, halfused_count,
+			subdriver->name, NUT_STRARG(halfused_names));
+	}
+
+	if (unused_names)
+		free(unused_names);
+	if (halfused_names)
+		free(halfused_names);
+
+	/* Check that all defined mappings were used? */
+	for (hidups_item = subdriver->hid2nut; hidups_item->info_type != NULL ; hidups_item++) {
+		known_mappings++;
+
+		if (hidups_item && hidups_item->hiddata
+		&& hidups_item->hiddata->mapping_handled
+		) {
+			used_mappings++;
+		}
+	}
+
+	/* We arbitrarily declare that having under 10 known or used
+	 * mappings is few enough to be loud about this */
+	upsdebugx( (known_mappings < 10 || used_mappings < 10) ? 0 : 5,
+		"%s: %" PRIuSIZE " mapping entries are defined, and "
+		"%" PRIuSIZE " were actually used from USB HID report, "
+		"in the selected NUT subdriver %s",
+		__func__, known_mappings, used_mappings,
+		subdriver->name);
+
+	if (known_mappings < 10 || used_mappings < 10)
+		upsdebugx(0, "Please check if there is a newer version of NUT available "
+			"(may be not packaged for your distribution yet), try a custom "
+			"build of development branch to test latest driver code per "
+			"%s/docs/user-manual.chunked/_installation_instructions.html#Installing_inplace, "
+			"and see %s/docs/developer-guide.chunked/new-drivers.html#hid-subdrivers "
+			"for suggestions how you can help improve this driver.",
+			NUT_WEBSITE_BASE, NUT_WEBSITE_BASE);
+}
+
 static subdriver_t *match_function_subdriver_name(int fatal_mismatch) {
 	char	*subdrv = getval("subdriver");
 	subdriver_t	*info = NULL;
@@ -836,15 +1022,15 @@ int instcmd(const char *cmdname, const char *extradata)
 		return instcmd("beeper.enable", NULL);
 	}
 
-	upsdebugx(1, "instcmd(%s, %s)",
-		cmdname,
-		extradata ? extradata : "[NULL]");
+	upsdebug_INSTCMD_STARTING(cmdname, extradata);
 
 	/* Retrieve and check netvar & item_path */
 	hidups_item = find_nut_info(cmdname);
 
 	/* Check for fallback if not found */
 	if (hidups_item == NULL) {
+		/* Process alias/fallback names */
+
 		upsdebugx(3, "%s: cmdname '%s' not found; "
 			"checking for alternatives",
 			__func__, cmdname);
@@ -900,7 +1086,9 @@ int instcmd(const char *cmdname, const char *extradata)
 			return instcmd("load.off.delay", dstate_getinfo("ups.delay.shutdown"));
 		}
 
-		upsdebugx(2, "instcmd: info element unavailable %s\n", cmdname);
+		/* FIXME: ..._UNKNOWN? */
+		upsdebugx(2, "instcmd: info element unavailable %s", cmdname);
+		upslog_INSTCMD_INVALID(cmdname, extradata);
 		return STAT_INSTCMD_INVALID;
 	}
 
@@ -911,33 +1099,62 @@ int instcmd(const char *cmdname, const char *extradata)
 
 	/* Check if the item is an instant command */
 	if (!(hidups_item->hidflags & HU_TYPE_CMD)) {
-		upsdebugx(2, "instcmd: %s is not an instant command\n", cmdname);
+		upsdebugx(2, "instcmd: %s is not an instant command", cmdname);
 		return STAT_INSTCMD_INVALID;
 	}
 
 	/* If extradata is empty, use the default value from the HID-to-NUT table */
 	val = extradata ? extradata : hidups_item->dfl;
-	if (!val) {
-		upsdebugx(2, "instcmd: %s requires an explicit parameter\n", cmdname);
+	if (!val && hidups_item->hidflags & HU_FLAG_PARAM_REQUIRED) {
+		upsdebugx(2, "instcmd: %s requires an explicit or default parameter", cmdname);
 		return STAT_INSTCMD_CONVERSION_FAILED;
 	}
 
 	/* Lookup the new value if needed */
 	if (hidups_item->hid2info != NULL) {
+		/* item->nuf() is expected to handle NULL if it must */
 		value = hu_find_valinfo(hidups_item->hid2info, val);
+		if (d_equal(value, -1) && errno == EINVAL) {
+			upsdebugx(2, "instcmd: %s does not support parameter %s",
+				cmdname, NUT_STRARG(val));
+			return STAT_INSTCMD_CONVERSION_FAILED;
+		}
 	} else {
-		value = atol(val);
+		if (!val) {
+			/* If we end up with atol(NULL) below, it should return
+			 * 0 on error anyway (on platforms where it would not
+			 * crash instead due to the NULL), so we make it portably
+			 * explicit here.
+			 */
+			/* FIXME: Look up data points (maybe via override.* or
+			 * default.* settings) for delay/etc. when handling
+			 * commands like shutdown.* or load.* ?
+			 */
+			upsdebugx(4, "instcmd: %s got no explicit nor default parameter, "
+				"but does not require one: falling back to 0", cmdname);
+			value = 0;
+		} else {
+			value = atol(val);
+		}
 	}
 
-	/* Actual variable setting */
+	/* Actual variable setting (as far as firmware is concerned) */
+	{	/* scoping + workaround for "error: the address of `argtmp` will always evaluate as `true`" */
+		char	argtmp[LARGEBUF], *s = NULL;
+		if (snprintf(argtmp, sizeof(argtmp), "%s (%f)",
+			NUT_STRARG(extradata), value) > 0)
+			s = argtmp;
+		upslog_INSTCMD_POWERSTATE_CHECKED(cmdname, s);
+	}
 	if (HIDSetDataValue(udev, hidups_item->hiddata, value) == 1) {
-		upsdebugx(3, "instcmd: SUCCEED\n");
+		upsdebugx(3, "instcmd: SUCCEED");
 		/* Set the status so that SEMI_STATIC vars are polled */
 		data_has_changed = TRUE;
 		return STAT_INSTCMD_HANDLED;
 	}
 
-	upsdebugx(3, "instcmd: FAILED\n"); /* TODO: HANDLED but FAILED, not UNKNOWN! */
+	/* upsdebugx(3, "instcmd: FAILED"); / * FIXME: return HANDLED but FAILED, not UNKNOWN! */
+	upslog_INSTCMD_FAILED(cmdname, extradata);
 	return STAT_INSTCMD_FAILED;
 }
 
@@ -947,51 +1164,79 @@ int setvar(const char *varname, const char *val)
 	hid_info_t	*hidups_item;
 	double		value;
 
-	upsdebugx(1, "setvar(%s, %s)", varname, val);
+	upsdebug_SET_STARTING(varname, val);
 
 	/* retrieve and check netvar & item_path */
 	hidups_item = find_nut_info(varname);
 
 	if (hidups_item == NULL) {
-		upsdebugx(2, "setvar: info element unavailable %s\n", varname);
+		upsdebugx(2, "setvar: info element unavailable %s", varname);
+		upslog_SET_UNKNOWN(varname, val);
 		return STAT_SET_UNKNOWN;
 	}
 
 	/* Checking item writability and HID Path */
 	if (!(hidups_item->info_flags & ST_FLAG_RW)) {
-		upsdebugx(2, "setvar: not writable %s\n", varname);
+		upsdebugx(2, "setvar: not writable %s", varname);
 		return STAT_SET_UNKNOWN;
 	}
 
 	/* handle server side variable */
 	if (hidups_item->hidflags & HU_FLAG_ABSENT) {
-		upsdebugx(2, "setvar: setting server side variable %s\n", varname);
+		upsdebugx(2, "setvar: setting server side variable %s", varname);
 		dstate_setinfo(hidups_item->info_type, "%s", val);
 		return STAT_SET_HANDLED;
 	}
 
 	/* HU_FLAG_ABSENT is the only case of HID Path == NULL */
 	if (hidups_item->hidpath == NULL) {
-		upsdebugx(2, "setvar: ID Path is NULL for %s\n", varname);
+		upsdebugx(2, "setvar: ID Path is NULL for %s", varname);
 		return STAT_SET_UNKNOWN;
 	}
 
+	/* FIXME: This code did not use "dfl"; should it start to?
+	 * If val is empty, use the default value from the HID-to-NUT table */
+	/* if (!val) val = hidups_item->dfl; */
+
+	if (!val && hidups_item->hidflags & HU_FLAG_PARAM_REQUIRED) {
+		upsdebugx(2, "setvar: %s requires an explicit or default parameter", varname);
+		return STAT_SET_CONVERSION_FAILED;
+	}
+
 	/* Lookup the new value if needed */
 	if (hidups_item->hid2info != NULL) {
+		/* item->nuf() is expected to handle NULL if it must */
 		value = hu_find_valinfo(hidups_item->hid2info, val);
+		if (d_equal(value, -1) && errno == EINVAL) {
+			upsdebugx(2, "setvar: %s does not support parameter %s",
+				varname, NUT_STRARG(val));
+			return STAT_SET_CONVERSION_FAILED;
+		}
 	} else {
-		value = atol(val);
+		if (!val) {
+			/* If we end up with atol(NULL) below, it should return
+			 * 0 on error anyway (on platforms where it would not
+			 * crash instead due to the NULL), so we make it portably
+			 * explicit here.
+			 */
+			upsdebugx(4, "setvar: %s got no explicit nor default parameter, "
+				"but does not require one: falling back to 0", varname);
+			value = 0;
+		} else {
+			value = atol(val);
+		}
 	}
 
 	/* Actual variable setting */
 	if (HIDSetDataValue(udev, hidups_item->hiddata, value) == 1) {
-		upsdebugx(5, "setvar: SUCCEED\n");
+		upsdebugx(5, "setvar: SUCCEED");
 		/* Set the status so that SEMI_STATIC vars are polled */
 		data_has_changed = TRUE;
 		return STAT_SET_HANDLED;
 	}
 
-	upsdebugx(3, "setvar: FAILED\n"); /* FIXME: HANDLED but FAILED, not UNKNOWN! */
+	/* upsdebugx(3, "setvar: FAILED"); / * FIXME: return HANDLED but FAILED, not UNKNOWN! */
+	upslog_SET_FAILED(varname, val);
 	return STAT_SET_UNKNOWN;
 }
 
@@ -1037,6 +1282,11 @@ void upsdrv_help(void)
 	printf("\n");
 }
 
+/* optionally tweak prognames[] entries */
+void upsdrv_tweak_prognames(void)
+{
+}
+
 void upsdrv_makevartable(void)
 {
 	char temp [MAX_STRING_SIZE];
@@ -1253,7 +1503,7 @@ void upsdrv_updateinfo(void)
 		ups_infoval_set(item, value);
 	}
 #ifdef DEBUG
-	upsdebugx(1, "took %.3f seconds handling interrupt reports...\n",
+	upsdebugx(1, "took %.3f seconds handling interrupt reports...",
 		interval());
 #endif
 	/* clear status buffer before beginning */
@@ -1289,7 +1539,7 @@ void upsdrv_updateinfo(void)
 
 	dstate_dataok();
 #ifdef DEBUG
-	upsdebugx(1, "took %.3f seconds handling feature reports...\n",
+	upsdebugx(1, "took %.3f seconds handling feature reports...",
 		interval());
 #endif
 }
@@ -1519,20 +1769,35 @@ void upsdrv_initups(void)
 			lbrb_log_delay_sec = ipv;
 		}
 	} else {
-		/* Activate APC BXnnnMI/BXnnnnMI tweaks, for details see
+		/* Activate APC BXnnnMI/BXnnnnMI/BVKnnnM2/BVKnnnnM2/BKnnnM2[_-]CH tweaks, for details see
 		 * https://github.com/networkupstools/nut/issues/2347
+		 * https://github.com/networkupstools/nut/issues/2565
+		 * https://github.com/networkupstools/nut/issues/2944
+		 * https://github.com/networkupstools/nut/issues/3006
 		 */
 		size_t	productLen = hd->Product ? strlen(hd->Product) : 0;
 
 		/* FIXME: Consider also ups.mfr.date as 2023 or newer?
 		 * Eventually up to some year this gets fixed?
 		 */
-		if (hd->Vendor
-		&&  productLen > 6	/* BXnnnMI at least */
-		&&  (!strcmp(hd->Vendor, "APC") || !strcmp(hd->Vendor, "American Power Conversion"))
-		&&  (strstr(hd->Product, " BX") || strstr(hd->Product, "BX") == hd->Product)
-		&&  (hd->Product[productLen - 2] == 'M' && hd->Product[productLen - 1] == 'I')
-		) {
+		if (hd->Vendor && productLen > 0
+		&& (!strcmp(hd->Vendor, "APC") || !strcmp(hd->Vendor, "American Power Conversion"))
+		&& ((
+			    productLen > 6	/* BXnnnMI at least */
+			&&  (strstr(hd->Product, " BX") || strstr(hd->Product, "BX") == hd->Product)
+			&&   hd->Product[productLen - 2] == 'M' && hd->Product[productLen - 1] == 'I'
+			) || (
+			    productLen > 7	/* BVKnnnM2 at least */
+			&&  (strstr(hd->Product, " BVK") || strstr(hd->Product, "BVK") == hd->Product)
+			&&   hd->Product[productLen - 2] == 'M' && hd->Product[productLen - 1] == '2'
+			) || (
+			    productLen > 9	/* BKnnnM2-CH at least, e.h. "Back-UPS BK650M2_CH" */
+			&&  (strstr(hd->Product, " BK") || strstr(hd->Product, "BK") == hd->Product)
+			&&   hd->Product[productLen - 5] == 'M' && hd->Product[productLen - 4] == '2'
+			&&  (hd->Product[productLen - 3] == '-' || hd->Product[productLen - 3] == '_')
+			&&   hd->Product[productLen - 2] == 'C' && hd->Product[productLen - 1] == 'H'
+			)
+		)) {
 			int	got_lbrb_log_delay_without_calibrating = testvar("lbrb_log_delay_without_calibrating") ? 1 : 0,
 				got_onlinedischarge_calibration = testvar("onlinedischarge_calibration") ? 1 : 0,
 				got_onlinedischarge_log_throttle_sec = testvar("onlinedischarge_log_throttle_sec") ? 1 : 0;
@@ -1589,10 +1854,23 @@ void upsdrv_initups(void)
 		lbrb_log_delay_without_calibrating = 1;
 	}
 
+	upsdebugx(1, "%s: Performing an initial UPS data walk with subdriver %s...",
+		__func__, subdriver->name);
 	if (hid_ups_walk(HU_WALKMODE_INIT) == FALSE) {
 		fatalx(EXIT_FAILURE, "Can't initialize data from HID UPS");
+	} else {
+		analyze_mapping_usage();
 	}
 
+	if (!ups_status)
+		upslogx(LOG_WARNING, "%s: No flag bits for 'ups.status' were explicitly reported; "
+			"it is possible a wrong 'subdriver' option was requested or detected "
+			"(in case of problems with device data, consider testing with other "
+			"explicit driver option 'subdriver' values)",
+			__func__);
+
+	upsdebugx(1, "%s: Optionally adjust some threshold values, if applicable and requested to...", __func__);
+
 	/* Set values below from user settings only if supported by UPS */
 	if (dstate_getinfo("battery.charge.low")) {
 		/* Retrieve user defined battery settings */
@@ -1636,6 +1914,8 @@ void upsdrv_initups(void)
 		}
 	}
 
+	upsdebugx(1, "%s: Optionally enable instant commands related to shutdown, if applicable...", __func__);
+
 	/* Enable instant commands below only if supported by UPS */
 	if (find_nut_info("load.off.delay")) {
 		/* Adds default with a delay value of '0' (= immediate) */
@@ -1652,6 +1932,8 @@ void upsdrv_initups(void)
 		dstate_addcmd("shutdown.return");
 		dstate_addcmd("shutdown.stayoff");
 	}
+
+	upsdebugx(1, "%s: finished", __func__);
 }
 
 void upsdrv_cleanup(void)
@@ -1684,10 +1966,11 @@ void possibly_supported(const char *mfr, HIDDevice_t *arghd)
 {
 	upsdebugx(0,
 "This %s device (%04x:%04x) is not (or perhaps not yet) supported\n"
-"by usbhid-ups. Please make sure you have an up-to-date version of NUT. If\n"
-"this does not fix the problem, try running the driver with the\n"
-"'-x productid=%04x' option. Please report your results to the NUT user's\n"
-"mailing list <nut-upsuser@lists.alioth.debian.org>.\n",
+"by usbhid-ups. Please make sure you have an up-to-date version of NUT.\n"
+"If this does not fix the problem, try running the driver with the\n"
+"'-x productid=%04x' option, or iterate with explicit '-x subdriver=...'\n"
+"option. Please report your results to the NUT user's mailing list\n"
+"at <nut-upsuser@lists.alioth.debian.org>.\n",
 	mfr, arghd->VendorID, arghd->ProductID, arghd->ProductID);
 
 	if (arghd->VendorID == 0x06da) {
@@ -1705,6 +1988,12 @@ static void process_boolean_info(const char *nutvalue)
 
 	upsdebugx(5, "process_boolean_info: %s", nutvalue);
 
+	/* Only neuter the other if we know the opposite to be true */
+	if (!strcmp(nutvalue, "online"))
+		process_boolean_info("!offline");
+	else if (!strcmp(nutvalue, "offline"))
+		process_boolean_info("!online");
+
 	if (*nutvalue == '!') {
 		nutvalue++;
 		clear = 1;
@@ -1793,15 +2082,19 @@ static int callback(
 	}
 
 	if (!subdriver) {
-		upsdebugx(1, "Manufacturer not supported!");
+		upsdebugx(1, "Manufacturer or model not supported!");
 		return 0;
 	}
 
-	upslogx(2, "Using subdriver: %s", subdriver->name);
+	upslogx(LOG_INFO, "Using subdriver: %s", subdriver->name);
 
 	if (subdriver->fix_report_desc(arghd, pDesc)) {
 		upsdebugx(2, "Report Descriptor Fixed");
 	}
+	upsdebugx(1, "%s: calling HIDDumpTree(); in case of problems with device data "
+		"please note that a wrong subdriver could have been chosen above; "
+		"consider testing others with an explicit driver option",
+		__func__);
 	HIDDumpTree(udev, arghd, subdriver->utab);
 
 #if !((defined SHUT_MODE) && SHUT_MODE)
@@ -1915,6 +2208,36 @@ static bool_t hid_ups_walk(walkmode_t mode)
 			if (item->hiddata == NULL)
 				continue;
 
+			/* Sanity-check that if we setvar() via mapping table
+			 * which refers us to a method, that such method entry
+			 * is not NULL (then we might get ERR READONLY later)
+			 */
+			if (ST_FLAG_RW && item->hid2info) {
+				if (
+					(item->hid2info->fun && !(item->hid2info->nuf) && (item->info_flags & ST_FLAG_RW))
+				||	(item->hid2info->nuf && !(item->hid2info->fun))
+				) {
+					/* Only one of these pointers is set - expecting
+					 * iteration for the other values until a sentinel
+					 * entry which then must exist; is it right there
+					 * in the next table line? */
+					if (item->hid2info[1].nut_value == NULL) {
+						/* The first line of that table is the
+						 * only meaningful one */
+						upsdebugx(1, "%s: WARNING: values for '%s' are handled with a mapping table "
+							"which only defines one dynamic method and can not let us %s "
+							"(or, technically it would map one value '%s'='%ld')."
+							"This may be a bug - please raise an issue with NUT maintainers "
+							"if this proves to be a practical problem with that data point.",
+							__func__, NUT_STRARG(item->info_type),
+							(item->hid2info->fun ? "write (setvar)" : "read"),
+							NUT_STRARG(item->hid2info->nut_value),
+							item->hid2info->hid_value
+						);
+					}
+				}
+			}
+
 			/* Special case for handling server side variables */
 			if (item->hidflags & HU_FLAG_ABSENT) {
 
@@ -1955,11 +2278,23 @@ static bool_t hid_ups_walk(walkmode_t mode)
 
 		case HU_WALKMODE_FULL_UPDATE:
 			/* These don't need polling after initinfo() */
-			if (item->hidflags & (HU_FLAG_ABSENT | HU_TYPE_CMD | HU_FLAG_STATIC))
+			if (item->hidflags & (HU_FLAG_ABSENT | HU_TYPE_CMD))
 				continue;
 
-			/* These need to be polled after user changes (setvar / instcmd) */
-			if ( (item->hidflags & HU_FLAG_SEMI_STATIC) && (data_has_changed == FALSE) )
+			/* These don't need polling after initinfo() normally
+			 * However in "pollonly" mode we use these to detect "Data stale"
+			 * condition (e.g. cable disconnected) by failing the reads:
+			 */
+			if ((item->hidflags & HU_FLAG_STATIC) && use_interrupt_pipe)
+				continue;
+
+			/* These need to be polled after user changes (setvar / instcmd)
+			 * or to detect "Data stale" in "pollonly" mode
+			 */
+			if (   (item->hidflags & HU_FLAG_SEMI_STATIC)
+				&& (data_has_changed == FALSE)
+				&& use_interrupt_pipe
+			)
 				continue;
 
 			break;
@@ -1985,7 +2320,7 @@ static bool_t hid_ups_walk(walkmode_t mode)
 			 */
 		default:
 			fatalx(EXIT_FAILURE, "hid_ups_walk: unknown update mode!");
-		}
+		}	/* end of: switch(mode) */
 #ifdef __clang__
 # pragma clang diagnostic pop
 #endif
@@ -2051,7 +2386,7 @@ static bool_t hid_ups_walk(walkmode_t mode)
 		case LIBUSB_ERROR_PIPE:      /* Broken pipe */
 		default:
 			/* Don't know what happened, try again later... */
-		   upsdebugx(1, "HIDGetDataValue unknown retcode '%i'", retcode);
+			upsdebugx(1, "HIDGetDataValue unknown retcode '%i'", retcode);
 			continue;
 		}
 
@@ -2074,7 +2409,7 @@ static bool_t hid_ups_walk(walkmode_t mode)
 		if (ups_infoval_set(item, value) != 1)
 			continue;
 
-		if (mode == HU_WALKMODE_INIT) {
+		if (mode == HU_WALKMODE_INIT || (!use_interrupt_pipe)) {
 			info_lkp_t	*info_lkp;
 
 			dstate_setflags(item->info_type, item->info_flags);
@@ -2090,8 +2425,11 @@ static bool_t hid_ups_walk(walkmode_t mode)
 			}
 
 			/* Loop on all existing values */
-			for (info_lkp = item->hid2info; info_lkp != NULL
-				&& info_lkp->nut_value != NULL; info_lkp++) {
+			for (
+				info_lkp = item->hid2info;
+				info_lkp != NULL && info_lkp->nut_value != NULL;
+				info_lkp++
+			) {
 				/* Check if this value is supported */
 				if (hu_find_infoval(item->hid2info, info_lkp->hid_value) != NULL) {
 					dstate_addenum(item->info_type, "%s", info_lkp->nut_value);
@@ -2187,11 +2525,6 @@ static void ups_alarm_set(void)
 	if (ups_status & STATUS(BYPASSMAN)) {
 		alarm_set("Manual bypass mode!");
 	}
-	if (ups_status & STATUS(ECOMODE)) {
-		buzzmode_set("vendor:default:ECO");
-		/* disable alarm for ECO as we don't want to raise alarm about it */
-		/* alarm_set("ECO(HE) mode!"); */
-	}
 }
 
 /* Return the current value of ups_status */
@@ -2273,9 +2606,35 @@ static void ups_status_set(void)
 		onlinedischarge_log_throttle_charge = -1;
 	}
 
-	if (!(ups_status & STATUS(ONLINE))) {
+	if ((ups_status & STATUS(OFFLINE))) {
 		status_set("OB");		/* on battery */
+		if ((ups_status & STATUS(ONLINE))) {
+			upsdebugx(1,
+				"%s: seems that UPS [%s] reports both online and on-battery "
+				"power states at the same time",
+				__func__, upsname);
+		}
+	}
+
+	if (!(ups_status & STATUS(ONLINE))) {
+		if ((ups_status & STATUS(OFFLINE))) {
+			status_set("OB");		/* on battery */
+		} else {
+			if ((ups_status & STATUS(DISCHRG))) {
+				upsdebugx(1,
+					"%s: seems that UPS [%s] does not report a power state, "
+					"but it is discharging - assuming it is on battery now",
+					__func__, upsname);
+
+				status_set("OB");
+			} else {
+				upsdebugx(1,
+					"%s: seems that UPS [%s] does not report a power state",
+					__func__, upsname);
+			}
+		}
 	} else if ((ups_status & STATUS(DISCHRG))) {
+		/* We are known to be online AND discharging... */
 		int	do_logmsg = 0, current_charge = 0;
 
 		/* if online but discharging */
@@ -2421,20 +2780,24 @@ static void ups_status_set(void)
 			__func__, upsname, msg_charge);
 		}
 	} else if ((ups_status & STATUS(ONLINE))) {
-		/* if simply online */
+		/* we get here if simply online, not discharging */
 		status_set("OL");
 	}
 
 	isCalibrating = status_get("CAL");
 
-	if ((ups_status & STATUS(DISCHRG)) &&
-		!(ups_status & STATUS(DEPLETED))) {
+	if ((ups_status & STATUS(DISCHRG))
+	&& !(ups_status & STATUS(DEPLETED))
+	) {
 		status_set("DISCHRG");		/* discharging */
 	}
-	if ((ups_status & STATUS(CHRG)) &&
-		!(ups_status & STATUS(FULLYCHARGED))) {
+
+	if ((ups_status & STATUS(CHRG))
+	&& !(ups_status & STATUS(FULLYCHARGED))
+	) {
 		status_set("CHRG");		/* charging */
 	}
+
 	if (ups_status & (STATUS(LOWBATT) | STATUS(TIMELIMITEXP) | STATUS(SHUTDOWNIMM))) {
 		if (lbrb_log_delay_sec < 1
 		|| (!isCalibrating && !lbrb_log_delay_without_calibrating)
@@ -2460,9 +2823,11 @@ static void ups_status_set(void)
 	} else {
 		last_lb_start = 0;
 	}
+
 	if (ups_status & STATUS(OVERLOAD)) {
 		status_set("OVER");		/* overload */
 	}
+
 	if ((ups_status & STATUS(REPLACEBATT)) || (ups_status & STATUS(NOBATTERY))) {
 		if (lbrb_log_delay_sec < 1
 		|| (!isCalibrating && !lbrb_log_delay_without_calibrating)
@@ -2489,25 +2854,31 @@ static void ups_status_set(void)
 	} else {
 		last_rb_start = 0;
 	}
+
 	if (ups_status & STATUS(TRIM)) {
 		status_set("TRIM");		/* SmartTrim */
 	}
+
 	if (ups_status & STATUS(BOOST)) {
 		status_set("BOOST");		/* SmartBoost */
 	}
+
 	if (ups_status & (STATUS(BYPASSAUTO) | STATUS(BYPASSMAN))) {
 		status_set("BYPASS");		/* on bypass */
 	}
+
 	if (ups_status & STATUS(ECOMODE)) {
 		buzzmode_set("vendor:default:ECO");	/* on ECO(HE) Mode,
 						 * should not happen
 						 * via ups.status anymore */
 	}
+
 	if (ups_status & STATUS(ESSMODE)) {
 		buzzmode_set("vendor:default:ESS");	/* on ESS Mode,
 						 * should not happen
 						 * via ups.status anymore */
 	}
+
 	if (ups_status & STATUS(OFF)) {
 		status_set("OFF");		/* ups is off */
 	}
@@ -2523,61 +2894,87 @@ static void ups_status_set(void)
 }
 
 /* find info element definition in info array
- * by NUT varname.
+ * by NUT varname, or NULL if not found.
  */
 static hid_info_t *find_nut_info(const char *varname)
 {
 	hid_info_t *hidups_item;
 
-	for (hidups_item = subdriver->hid2nut; hidups_item->info_type != NULL ; hidups_item++) {
+	if (!varname) {
+		upsdebugx(2, "%s: varname == NULL", __func__);
+		errno = EINVAL;
+		return NULL;
+	}
 
+	if (!*varname) {
+		upsdebugx(2, "%s: varname is an empty string", __func__);
+		errno = EINVAL;
+		return NULL;
+	}
+
+	for (hidups_item = subdriver->hid2nut; hidups_item->info_type != NULL ; hidups_item++) {
 		if (strcasecmp(hidups_item->info_type, varname))
 			continue;
 
-		if (hidups_item->hiddata != NULL)
+		if (hidups_item->hiddata != NULL) {
+			errno = 0;
 			return hidups_item;
+		}
 	}
 
-	upsdebugx(2, "find_nut_info: unknown info type: %s", varname);
+	upsdebugx(2, "%s: unknown info type: %s", __func__, varname);
+	errno = EINVAL;
 	return NULL;
 }
 
 /* find info element definition in info array
- * by HID data pointer.
+ * by HID data pointer, or NULL if not found.
  */
 static hid_info_t *find_hid_info(const HIDData_t *hiddata)
 {
 	hid_info_t *hidups_item;
 
-	if(!hiddata) {
+	if (!hiddata) {
 		upsdebugx(2, "%s: hiddata == NULL", __func__);
+		errno = EINVAL;
 		return NULL;
 	}
 
 	for (hidups_item = subdriver->hid2nut; hidups_item->info_type != NULL ; hidups_item++) {
-
 		/* Skip server side vars */
 		if (hidups_item->hidflags & HU_FLAG_ABSENT)
 			continue;
 
-		if (hidups_item->hiddata == hiddata)
+		if (hidups_item->hiddata == hiddata) {
+			errno = 0;
 			return hidups_item;
+		}
 	}
 
+	errno = EINVAL;
 	return NULL;
 }
 
-/* find the HID Item value matching that NUT value */
-/* useful for set with value lookup... */
+/* Find the HID Item value matching that NUT value;
+ * useful for setvar() with a value lookup...
+ * This method (and hid2info->nuf(value)) can return
+ * a long value to be set, or "-1" AND errno==EINVAL
+ * to propagate an error.
+ */
 static long hu_find_valinfo(info_lkp_t *hid2info, const char* value)
 {
 	info_lkp_t	*info_lkp;
 
+	errno = 0;
+
 	/* if a conversion function is defined, use 'value' as argument for it */
 	if (hid2info->nuf != NULL) {
 		double	hid_value;
 		hid_value = hid2info->nuf(value);
-		upsdebugx(5, "hu_find_valinfo: found %g (value: %s)", hid_value, value);
+		upsdebugx(5, "hu_find_valinfo: found %g (value: '%s'%s)",
+			hid_value, value,
+			(errno == EINVAL ? ", invalid input" : "")
+		);
 		return hid_value;
 	}
 
@@ -2593,14 +2990,22 @@ static long hu_find_valinfo(info_lkp_t *hid2info, const char* value)
 	upsdebugx(3,
 		"hu_find_valinfo: no matching HID value for this INFO_* value (%s)",
 		value);
+	errno = EINVAL;
 	return -1;
 }
 
-/* find the NUT value matching that HID Item value */
+/* find the NUT value matching that HID Item value
+ * This method (and hid2info->fun(value)) can return
+ * a string value to be set, or NULL AND errno==EINVAL
+ * to propagate an error (in practice, a NULL suffices
+ * to not-set a dstate).
+ */
 static const char *hu_find_infoval(info_lkp_t *hid2info, const double value)
 {
 	info_lkp_t	*info_lkp;
 
+	errno = 0;
+
 	/* if a conversion function is defined, use 'value' as argument for it */
 	if (hid2info->fun != NULL) {
 		return hid2info->fun(value);
@@ -2619,22 +3024,40 @@ static const char *hu_find_infoval(info_lkp_t *hid2info, const double value)
 	upsdebugx(3,
 		"hu_find_infoval: no matching INFO_* value for this HID value (%g)",
 		value);
+	errno = EINVAL;
 	return NULL;
 }
 
 /* return -1 on failure, 0 for a status update and 1 in all other cases */
 static int ups_infoval_set(hid_info_t *item, double value)
 {
-	const char	*nutvalue;
+	const char	*nutvalue = NULL;
+	const char	*pType = (nut_debug_level > 0 && item->hiddata) ? HIDDataType(item->hiddata) : NULL;
 
 	/* need lookup'ed translation? */
-	if (item->hid2info != NULL){
-
+	if (item->hid2info != NULL) {
 		if ((nutvalue = hu_find_infoval(item->hid2info, value)) == NULL) {
-			upsdebugx(5, "Lookup [%g] failed for [%s]", value, item->info_type);
+			upsdebugx(5, "%s: Lookup [%g] failed for [%s]", __func__, value, item->info_type);
 			return -1;
 		}
+		/* clause continued below after this message... */
+	}
 
+	if (item->hiddata != NULL) {
+		if (!item->hiddata->mapping_handled) {
+			upsdebugx(5, "%s: setting report descriptor mapping for '%s' (%s) as handled",
+				__func__, NUT_STRARG(item->hidpath), NUT_STRARG(pType));
+			item->hiddata->mapping_handled = TRUE;
+		} else {
+			upsdebugx(5, "%s: report descriptor mapping for '%s' (%s) was already set as handled",
+				__func__, NUT_STRARG(item->hidpath), NUT_STRARG(pType));
+		}
+	} else {
+		upsdebugx(5, "%s: got no report descriptor mapping for '%s'",
+			__func__, NUT_STRARG(item->hidpath));
+	}
+
+	if (nutvalue != NULL) {
 		/* deal with boolean items */
 		if (!strncmp(item->info_type, "BOOL", 4)) {
 			process_boolean_info(nutvalue);
@@ -2649,19 +3072,7 @@ static int ups_infoval_set(hid_info_t *item, double value)
 
 		dstate_setinfo(item->info_type, "%s", nutvalue);
 	} else {
-#ifdef HAVE_PRAGMAS_FOR_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
-#pragma GCC diagnostic push
-#endif
-#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
-#pragma GCC diagnostic ignored "-Wformat-nonliteral"
-#endif
-#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_FORMAT_SECURITY
-#pragma GCC diagnostic ignored "-Wformat-security"
-#endif
-		dstate_setinfo(item->info_type, item->dfl, value);
-#ifdef HAVE_PRAGMAS_FOR_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
-#pragma GCC diagnostic pop
-#endif
+		dstate_setinfo_dynamic(item->info_type, item->dfl, "%f", value);
 	}
 
 	return 1;
