@@ -32,7 +32,7 @@
 #include "serial.h"
 
 #define DRIVER_NAME	"GE/IMV/Victron UPS driver"
-#define DRIVER_VERSION	"0.24"
+#define DRIVER_VERSION	"0.26"
 
 /* driver description structure */
 upsdrv_info_t upsdrv_info = {
@@ -100,8 +100,13 @@ static int instcmd(const char *cmdname, const char *extra)
 {
 	char temp[ LENGTH_TEMP ];
 
+	/* May be used in logging below, but not as a command argument */
+	NUT_UNUSED_VARIABLE(extra);
+	upsdebug_INSTCMD_STARTING(cmdname, extra);
+
 	if(!strcasecmp(cmdname, "calibrate.start"))
 	{
+		upslog_INSTCMD_POWERSTATE_MAYBE(cmdname, extra);
 		if(get_data("vTi5!",temp))
 		{
 			upsdebugx(1, "instcmd: ser_send calibrate.start failed");
@@ -116,6 +121,7 @@ static int instcmd(const char *cmdname, const char *extra)
 	}
 	else if(!strcasecmp(cmdname, "calibrate.stop"))
 	{
+		upslog_INSTCMD_POWERSTATE_MAYBE(cmdname, extra);
 		if(get_data("vTi2!",temp))
 		{
 			upsdebugx(1, "instcmd: ser_send calibrate.stop failed");
@@ -129,6 +135,7 @@ static int instcmd(const char *cmdname, const char *extra)
 	}
 	else if(!strcasecmp(cmdname, "test.battery.stop"))
 	{
+		upslog_INSTCMD_POWERSTATE_MAYBE(cmdname, extra);
 		if(get_data("vTi2!",temp))
 		{
 			upsdebugx(1, "instcmd: ser_send test.battery.stop failed");
@@ -142,6 +149,7 @@ static int instcmd(const char *cmdname, const char *extra)
 	}
 	else if(!strcasecmp(cmdname, "test.battery.start"))
 	{
+		upslog_INSTCMD_POWERSTATE_MAYBE(cmdname, extra);
 		if(get_data("vTi4!",temp))
 		{
 			upsdebugx(1, "instcmd: ser_send test.battery.start failed");
@@ -183,6 +191,7 @@ static int instcmd(const char *cmdname, const char *extra)
 	}
 	else if(!strcasecmp(cmdname, "bypass.stop"))
 	{
+		upslog_INSTCMD_POWERSTATE_MAYBE(cmdname, extra);
 		if(get_data("vTi2!",temp))
 		{
 			upsdebugx(1, "instcmd: ser_send bypass.stop failed");
@@ -196,6 +205,7 @@ static int instcmd(const char *cmdname, const char *extra)
 	}
 	else if(!strcasecmp(cmdname, "bypass.start"))
 	{
+		upslog_INSTCMD_POWERSTATE_MAYBE(cmdname, extra);
 		if(get_data("vTi101!",temp))
 		{
 			upsdebugx(1, "instcmd: ser_send bypass.start failed");
@@ -210,7 +220,7 @@ static int instcmd(const char *cmdname, const char *extra)
 	}
 	else
 	{
-		upsdebugx(1, "instcmd: unknown command: [%s] [%s]", cmdname, extra);
+		upslog_INSTCMD_UNKNOWN(cmdname, extra);
 		return STAT_INSTCMD_UNKNOWN;
 	}
 }
@@ -511,6 +521,11 @@ void upsdrv_help(void)
 {
 }
 
+/* optionally tweak prognames[] entries */
+void upsdrv_tweak_prognames(void)
+{
+}
+
 /* list flags and values that you want to receive via -x */
 void upsdrv_makevartable(void)
 {
