@@ -9,7 +9,7 @@
 # Subject to same license as the NUT Project.
 #
 # Copyright (C)
-#	2022 Jim Klimov <jimklimov+nut@gmail.com>
+#	2022-2025 Jim Klimov <jimklimov+nut@gmail.com>
 #
 
 if [ $# = 2 ] && [ -s "$1" ] && [ -s "$2" ]; then
@@ -19,6 +19,10 @@ else
     exit 1
 fi
 
+# tools
+[ -n "${GREP}" ] || { GREP="`command -v grep`" && [ x"${GREP}" != x ] || { echo "$0: FAILED to locate GREP tool" >&2 ; exit 1 ; } ; }
+[ -n "${EGREP}" ] || { if ( [ x"`echo a | $GREP -E '(a|b)'`" = xa ] ) 2>/dev/null ; then EGREP="$GREP -E" ; else EGREP="`command -v egrep`" ; fi && [ x"${EGREP}" != x ] || { echo "$0: FAILED to locate EGREP tool" >&2 ; exit 1 ; } ; }
+
 # Pre-sort just in case:
 DATA1="`sort -n < "$1"`"
 DATA2="`sort -n < "$2"`"
@@ -28,8 +32,8 @@ DATA2="`sort -n < "$2"`"
 # or multi-digit numbers without a decimal point (assuming
 # differences in shorter numbers or counters may be important)
 diff -bu <(echo "$DATA1") <(echo "$DATA2") \
-| grep -E '^[+-][^+-]' \
-| grep -vE '^[^:]*(power|load|voltage|current|frequency|temperature|humidity): ([0-9][0-9]*|[0-9][0-9]*\.[0-9][0-9]*)$'
+| ${EGREP} '^[+-][^+-]' \
+| ${EGREP} -v '^[^:]*(power|load|voltage|current|frequency|temperature|humidity): ([0-9][0-9]*|[0-9][0-9]*\.[0-9][0-9]*)$'
 
 # Note: up to user to post-filter, "^driver.version.*:"
 # may be deemed irrelevant as well
